@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007 Fabrice Colin
+ *  Copyright 2007-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +16,9 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <stdio.h>
+#include <stdlib.h>
 #include <iostream>
-#include <cstdlib>
 
 #include "Languages.h"
 #include "DBusIndex.h"
@@ -30,7 +31,7 @@ using std::set;
 using std::map;
 using std::min;
 
-static const char *g_fieldNames[] = { "caption", "url", "type", "language", "modtime", "size", "extract", NULL };
+static const char *g_fieldNames[] = { "caption", "url", "type", "language", "modtime", "size", "extract", "score", NULL };
 
 static DBusGConnection *getBusConnection(void)
 {
@@ -57,8 +58,8 @@ static DBusGProxy *getBusProxy(DBusGConnection *pBus)
 		return NULL;
 	}
 
-	return dbus_g_proxy_new_for_name(pBus,
-		"de.berlios.Pinot", "/de/berlios/Pinot", "de.berlios.Pinot");
+	return dbus_g_proxy_new_for_name(pBus, PINOT_DBUS_SERVICE_NAME,
+		PINOT_DBUS_OBJECT_PATH, PINOT_DBUS_SERVICE_NAME);
 }
 
 DBusIndex::DBusIndex(IndexInterface *pROIndex) :
@@ -192,6 +193,10 @@ bool DBusIndex::documentInfoFromDBus(DBusMessageIter *iter, unsigned int &docId,
 		{
 			docInfo.setExtract(pValue);
 		}
+		else if (fieldName == g_fieldNames[7])
+		{
+			docInfo.setScore((float)atof(pValue));
+		}
 	}
 	while (dbus_message_iter_next(&array_iter));
 
@@ -230,7 +235,7 @@ bool DBusIndex::documentInfoToDBus(DBusMessageIter *iter, unsigned int docId,
 	for (unsigned int fieldNum = 0; g_fieldNames[fieldNum] != NULL; ++fieldNum)
 	{
 		string value;
-		char sizeStr[64];
+		char numStr[64];
 
 		switch (fieldNum)
 		{
@@ -238,7 +243,7 @@ bool DBusIndex::documentInfoToDBus(DBusMessageIter *iter, unsigned int docId,
 				value = docInfo.getTitle();
 				break;
 			case 1:
-				value = docInfo.getLocation();
+				value = docInfo.getLocation(true);
 				break;
 			case 2:
 				value = docInfo.getType();
@@ -250,12 +255,16 @@ bool DBusIndex::documentInfoToDBus(DBusMessageIter *iter, unsigned int docId,
 				value = docInfo.getTimestamp();
 				break;
 			case 5:
-				snprintf(sizeStr, 64, "%u", docInfo.getSize());
-				value = sizeStr;
+				snprintf(numStr, 64, "%u", docInfo.getSize());
+				value = numStr;
 				break;
 			case 6:
 				value = docInfo.getExtract();
 				break;
+			case 7:
+				snprintf(numStr, 64, "%f", docInfo.getScore());
+				value = numStr;
+				break;
 			default:
 				break;
 		}
@@ -489,7 +498,7 @@ bool DBusIndex::getDocumentTerms(unsigned int docId,
 }
 
 /// Sets the list of known labels.
-bool DBusIndex::setLabels(const set<string> &labels)
+bool DBusIndex::setLabels(const set<string> &labels, bool resetLabels)
 {
 	// Not allowed here
 	return false;
@@ -591,52 +600,6 @@ bool DBusIndex::addLabel(const string &name)
 	return addedLabel;
 }
 
-/// Renames a label.
-bool DBusIndex::renameLabel(const string &name, const string &newName)
-{
-	bool renamedLabel = false;
-
-	DBusGConnection *pBus = getBusConnection();
-	if (pBus == NULL)
-	{
-		return false;
-	}
-
-	DBusGProxy *pBusProxy = getBusProxy(pBus);
-	if (pBusProxy == NULL)
-	{
-		cerr << "DBusIndex::renameLabel: couldn't get bus proxy" << endl;
-		return false;
-	}
-
-	GError *pError = NULL;
-	const char *pOldLabel = name.c_str();
-	const char *pNewLabel = newName.c_str();
-
-	if (dbus_g_proxy_call(pBusProxy, "RenameLabel", &pError,
-		G_TYPE_STRING, pOldLabel,
-		G_TYPE_STRING, pNewLabel,
-		G_TYPE_INVALID,
-		G_TYPE_STRING, &pNewLabel,
-		G_TYPE_INVALID) == TRUE)
-	{
-		renamedLabel = true;
-	}
-	else
-	{
-		if (pError != NULL)
-		{
-			cerr << "DBusIndex::renameLabel: " << pError->message << endl;
-			g_error_free(pError);
-		}
-	}
-
-	g_object_unref(pBusProxy);
-	// FIXME: don't we have to call dbus_g_connection_unref(pBus); ?
-
-	return renamedLabel;
-}
-
 /// Deletes all references to a label.
 bool DBusIndex::deleteLabel(const string &name)
 {
@@ -1066,8 +1029,8 @@ bool DBusIndex::updateDocumentInfo(unsigned int docId, const DocumentInfo &docIn
 	}
 
 	// FIXME: AFAIK we can't use DBusGProxy with message iterators
-	DBusMessage *pMsg = dbus_message_new_method_call("de.berlios.Pinot",
-		"/de/berlios/Pinot", "de.berlios.Pinot", "SetDocumentInfo");
+	DBusMessage *pMsg = dbus_message_new_method_call(PINOT_DBUS_SERVICE_NAME,
+		PINOT_DBUS_OBJECT_PATH, PINOT_DBUS_SERVICE_NAME, "SetDocumentInfo");
 	if (pMsg == NULL)
 	{
 		cerr << "DBusIndex::updateDocumentInfo: couldn't call method" << endl;
