@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -23,6 +23,7 @@
 #include <utility>
 
 #include "Document.h"
+#include "StringManip.h"
 #include "CJKVTokenizer.h"
 #include "QueryProperties.h"
 
@@ -68,9 +69,7 @@ class FilterRemover : public Dijon::CJKVTokenizer::TokensHandler
 							m_freeQuery.erase(tokPosStart);
 						}
 					}
-#ifdef DEBUG
-					cout << "QueryProperties::getFreeQuery: removed " << tok << "..." << endl;
-#endif
+
 					return true;
 				}
 			}
@@ -109,6 +108,7 @@ class SetTokensHandler : public Dijon::CJKVTokenizer::TokensHandler
 QueryProperties::QueryProperties() :
 	m_type(XAPIAN_QP),
 	m_order(RELEVANCE),
+	m_diacriticSensitive(false),
 	m_resultsCount(10),
 	m_indexResults(NOTHING),
 	m_modified(false)
@@ -120,6 +120,7 @@ QueryProperties::QueryProperties(const string &name, const string &freeQuery,
 	m_name(name),
 	m_type(type),
 	m_order(RELEVANCE),
+	m_diacriticSensitive(false),
 	m_freeQuery(freeQuery),
 	m_resultsCount(10),
 	m_indexResults(NOTHING),
@@ -133,6 +134,7 @@ QueryProperties::QueryProperties(const QueryProperties &other) :
 	m_type(other.m_type),
 	m_order(other.m_order),
 	m_language(other.m_language),
+	m_diacriticSensitive(other.m_diacriticSensitive),
 	m_freeQuery(other.m_freeQuery),
 	m_freeQueryWithoutFilters(other.m_freeQueryWithoutFilters),
 	m_resultsCount(other.m_resultsCount),
@@ -154,6 +156,7 @@ QueryProperties &QueryProperties::operator=(const QueryProperties &other)
 		m_type = other.m_type;
 		m_order = other.m_order;
 		m_language = other.m_language;
+		m_diacriticSensitive = other.m_diacriticSensitive;
 		m_freeQuery = other.m_freeQuery;
 		m_freeQueryWithoutFilters = other.m_freeQueryWithoutFilters;
 		m_resultsCount = other.m_resultsCount;
@@ -189,6 +192,7 @@ void QueryProperties::removeFilters(void)
 {
 	m_freeQueryWithoutFilters.clear();
 
+	StringManip::trimSpaces(m_freeQuery);
 	if (m_freeQuery.empty() == true)
 	{
 		return;
@@ -200,11 +204,19 @@ void QueryProperties::removeFilters(void)
 		return;
 	}
 
+	// Remove carriage returns
+	string noCR(StringManip::replaceSubString(m_freeQuery, "\r\n", "\n"));
+	// If there's a line break right after an hyphen, remove both
+	string dehyphenedOnNL(StringManip::replaceSubString(noCR, "-\n", ""));
+	// ... and replace line breaks with spaces
+	m_freeQuery = StringManip::replaceSubString(dehyphenedOnNL, "\n", " ");
+
 	Dijon::CJKVTokenizer tokenizer;
 	FilterRemover handler(m_freeQuery);
 
 	tokenizer.tokenize(m_freeQuery, handler, true);
 	m_freeQueryWithoutFilters = handler.m_freeQuery;
+	StringManip::trimSpaces(m_freeQueryWithoutFilters);
 }
 
 /// Sets the name.
@@ -255,6 +267,18 @@ string QueryProperties::getStemmingLanguage(void) const
 	return m_language;
 }
 
+/// Sets whether the query is sensitive to diacritics.
+void QueryProperties::setDiacriticSensitive(bool sensitive)
+{
+	m_diacriticSensitive = sensitive;
+}
+
+/// Gets whether the query is sensitive to diacritics.
+bool QueryProperties::getDiacriticSensitive(void) const
+{
+	return m_diacriticSensitive;
+}
+
 /// Sets the query string.
 void QueryProperties::setFreeQuery(const string &freeQuery)
 {
