@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,17 +16,20 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <string.h>
 #include <strings.h>
+#ifdef HAVE_REGEX_H
 #include <regex.h>
+#endif
 #include <stdio.h>
 #include <sstream>
 #include <iostream>
-#include <cstring>
 
 #include "StringManip.h"
 #include "TimeConverter.h"
@@ -158,7 +161,8 @@ void XapianDatabase::openDatabase(void)
 	// Should we build the spelling database ?
 	char *pEnvVar = getenv("PINOT_SPELLING_DB");
 	if ((pEnvVar != NULL) &&
-		(strncasecmp(pEnvVar, "no", 2) == 0))
+		(strlen(pEnvVar) > 0) &&
+		(strncasecmp(pEnvVar, "N", 1) == 0))
 	{
 		// No
 		m_withSpelling = false;
@@ -187,6 +191,7 @@ void XapianDatabase::openDatabase(void)
 	{
 		Url urlObj(m_databaseName);
 
+		// FIXME: in newer versions, the remote backend supports writing
 		if (m_readOnly == false)
 		{
 			cerr << "XapianDatabase::openDatabase: remote databases " << m_databaseName << " are read-only" << endl;
@@ -271,8 +276,12 @@ void XapianDatabase::openDatabase(void)
 			<< " doesn't exist" << endl;
 #endif
 
-		// Database directory doesn't exist, create it (mode 755)
-		if (mkdir(m_databaseName.c_str(), (mode_t)S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH) != 0)
+		// Database directory doesn't exist, create it
+#ifdef WIN32
+		if (mkdir(m_databaseName.c_str()) != 0)
+#else
+		if (mkdir(m_databaseName.c_str(), (mode_t)(S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)) != 0)
+#endif
 		{
 			cerr << "XapianDatabase::openDatabase: couldn't create database directory "
 				<< m_databaseName << endl;
@@ -368,6 +377,25 @@ bool XapianDatabase::isOpen(void) const
 	return m_isOpen;
 }
 
+/// Returns true if the database is a merge of other databases.
+bool XapianDatabase::isMerge(void) const
+{
+	return m_merge;
+}
+
+/// Returns false if the database isn't opened in write mode.
+bool XapianDatabase::isWritable(void) const
+{
+	if ((m_isOpen == false) ||
+		(m_readOnly == true) ||
+		(m_merge == true))
+	{
+		return false;
+	}
+
+	return true;
+}
+
 /// Returns false if the database was of an obsolete format.
 bool XapianDatabase::wasObsoleteFormat(void) const
 {
@@ -378,7 +406,6 @@ bool XapianDatabase::wasObsoleteFormat(void) const
 void XapianDatabase::reopen(void)
 {
 	// This is provided by Xapian::Database
-	// FIXME: get the write lock to make sure read operations are not in progress ?
 	if (pthread_mutex_lock(&m_rwLock) == 0)
 	{
 		if (m_pDatabase != NULL)
@@ -426,14 +453,15 @@ Xapian::Database *XapianDatabase::readLock(void)
 			// Lock both indexes
 			Xapian::Database *pFirstDatabase = m_pFirst->readLock();
 			Xapian::Database *pSecondDatabase = m_pSecond->readLock();
-			// Copy the first one
-			m_pDatabase = new Xapian::Database(*pFirstDatabase);
-			// Add the second index to it
-			if (pSecondDatabase != NULL)
+			if ((pFirstDatabase != NULL) &&
+				(pSecondDatabase != NULL))
 			{
+				// Copy the first one
+				m_pDatabase = new Xapian::Database(*pFirstDatabase);
+				// Add the second index to it
 				m_pDatabase->add_database(*pSecondDatabase);
+				// Until unlock() is called, both indexes are read locked
 			}
-			// Until unlock() is called, both indexes are read locked
 
 			return m_pDatabase;
 		}
@@ -506,13 +534,14 @@ void XapianDatabase::unlock(void)
 
 bool XapianDatabase::badRecordField(const string &field)
 {
+	bool isBadField = false;
+#ifdef HAVE_REGEX_H
 	regex_t fieldRegex;
 	regmatch_t pFieldMatches[1];
-	bool isBadField = false;
 
 	// A bad field is one that includes one of our field delimiters
 	if (regcomp(&fieldRegex,
-		"(url|sample|caption|type|modtime|language|size)=",
+		"(url|ipath|sample|caption|type|modtime|language|size)=",
 		REG_EXTENDED|REG_ICASE) == 0)
 	{
 		if (regexec(&fieldRegex, field.c_str(), 1,
@@ -522,6 +551,20 @@ bool XapianDatabase::badRecordField(const string &field)
 		}
 	}
 	regfree(&fieldRegex);
+#else
+	// A bad field is one that includes one of our field delimiters
+	if ((field.find("url=") != string::npos) ||
+		(field.find("ipath=") != string::npos) ||
+		(field.find("sample=") != string::npos) ||
+		(field.find("caption=") != string::npos) ||
+		(field.find("type=") != string::npos) ||
+		(field.find("modtime=") != string::npos) ||
+		(field.find("language=") != string::npos) ||
+		(field.find("size=") != string::npos))
+	{
+		isBadField = true;
+	}
+#endif
 
 	return isBadField;
 }
@@ -541,6 +584,8 @@ string XapianDatabase::propsToRecord(DocumentInfo *pDoc)
 
 	// Set the document data omindex-style
 	record += pDoc->getLocation();
+	record += "\nipath=";
+	record += Url::escapeUrl(pDoc->getInternalPath());
 	// The sample will be generated at query time
 	record += "\nsample=";
 	record += "\ncaption=";
@@ -597,6 +642,13 @@ void XapianDatabase::recordToProps(const string &record, DocumentInfo *pDoc)
 		url = Url::canonicalizeUrl(url);
 	}
 	pDoc->setLocation(url);
+	// Get the internal path 
+	string ipath(StringManip::extractField(record, "ipath=", "\n"));
+	if (ipath.empty() == false)
+	{
+		ipath = Url::unescapeUrl(ipath);
+	}
+	pDoc->setInternalPath(ipath);
 	// Get the type
 	pDoc->setType(StringManip::extractField(record, "type=", "\n"));
 	// ... the language, if available
