@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005-2008 Fabrice Colin
+ *  Copyright 2005-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -137,34 +137,141 @@ class TimeValueRangeProcessor : public Xapian::ValueRangeProcessor
 
 };
 
-class PrefixDecider : public Xapian::ExpandDecider
+class TermDecider : public Xapian::ExpandDecider
 {
 	public:
-		PrefixDecider(const string &allowedPrefixes) :
+		TermDecider(Xapian::Database *pIndex,
+			Xapian::Stem *pStemmer,
+			Xapian::Stopper *pStopper,
+			const string &allowedPrefixes,
+			Xapian::Query &query) :
 			Xapian::ExpandDecider(),
-			m_allowedPrefixes(allowedPrefixes)
+			m_pIndex(pIndex),
+			m_pStemmer(pStemmer),
+			m_pStopper(pStopper),
+			m_allowedPrefixes(allowedPrefixes),
+			m_pTermsToAvoid(NULL)
 		{
+			m_pTermsToAvoid = new set<string>();
+
+			for (Xapian::TermIterator termIter = query.get_terms_begin();
+				termIter != query.get_terms_end(); ++termIter)
+			{
+				string term(*termIter);
+
+				if (isupper((int)(term[0])) == 0)
+				{
+					m_pTermsToAvoid->insert(term);
+					if (m_pStemmer != NULL)
+					{
+						string stem((*m_pStemmer)(term));
+						m_pTermsToAvoid->insert(stem);
+					}
+				}
+				else if (term[0] == 'Z')
+				{
+					m_pTermsToAvoid->insert(term.substr(1));
+				}
+			}
+#ifdef DEBUG
+			cout << "TermDecider: avoiding " << m_pTermsToAvoid->size() << " terms" << endl;
+#endif
 		}
-		~PrefixDecider()
+		~TermDecider()
 		{
+			if (m_pTermsToAvoid != NULL)
+			{
+				delete m_pTermsToAvoid;
+			}
 		}
 
 		virtual bool operator()(const std::string &term) const
 		{
-			if ((isupper((int)(term[0])) == 0) ||
-				(m_allowedPrefixes.find(term[0]) != string::npos))
+			CJKVTokenizer tokenizer;
+			bool isPrefixed = false;
+
+			// Reject short terms
+			if ((tokenizer.has_cjkv(term) == false) &&
+				(term.length() < 3))
+			{
+				return false;
+			}
+
+			// Reject terms with prefixes we don't want
+			if (isupper((int)(term[0])) != 0)
+			{
+				isPrefixed = true;
+
+				if (m_allowedPrefixes.find(term[0]) == string::npos)
+				{
+					return false;
+				}
+			}
+
+			// Reject terms with spaces
+			if (term.find_first_of(" \t\r\n") != string::npos)
+			{
+				return false;
+			}
+
+			// Reject terms that occur only once
+			if ((m_pIndex != NULL) &&
+				(m_pIndex->get_termfreq(term) <= 1))
+			{
+				return false;
+			}
+
+			// Reject stop words
+			if ((m_pStopper != NULL) &&
+				((*m_pStopper)(term) == true))
+			{
+				return false;
+			}
+
+			// Stop here if there's no specific terms to avoid
+			if (m_pTermsToAvoid->empty() == true)
 			{
 				return true;
 			}
-#ifdef DEBUG
-			cout << "PrefixDecider::operator: rejecting " << term << endl;
-#endif
 
-			return false;
+			// Reject query terms
+			if (m_pTermsToAvoid->find(term) != m_pTermsToAvoid->end())
+			{
+				return false;
+			}
+
+			// Stop here is there's no stemmer
+			if (m_pStemmer == NULL)
+			{
+				return true;
+			}
+
+			// Reject terms that stem to the same as query terms
+			// or previously validated terms
+			string stem;
+			if (isPrefixed == true)
+			{
+				stem = (*m_pStemmer)(term.substr(1));
+			}
+			else
+			{
+				stem = (*m_pStemmer)(term);
+			}
+			if (m_pTermsToAvoid->find(stem) != m_pTermsToAvoid->end())
+			{
+				return false;
+			}
+			m_pTermsToAvoid->insert(stem);
+
+			return true;
 		}
 
 	protected:
+		Xapian::Database *m_pIndex;
+		Xapian::Stem *m_pStemmer;
+		Xapian::Stopper *m_pStopper;
 		string m_allowedPrefixes;
+		set<string> *m_pTermsToAvoid;
 
 };
 
@@ -466,6 +573,9 @@ class QueryModifier : public Dijon::CJKVTokenizer::TokensHandler
 XapianEngine::XapianEngine(const string &database) :
 	SearchEngineInterface()
 {
+	// We expect documents to have been converted to UTF-8 at indexing time
+	m_charset = "UTF-8";
+
 	// If the database name ends with a slash, remove it
 	if (database[database.length() - 1] == '/')
 	{
@@ -486,7 +596,6 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 	const string &limitQuery, string &correctedFreeQuery, bool minimal)
 {
 	Xapian::QueryParser parser;
-	Xapian::Stem stemmer;
 	CJKVTokenizer tokenizer;
 	string freeQuery(queryProps.getFreeQuery());
 	unsigned int tokensCount = 1;
@@ -543,26 +652,13 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 	if ((minimal == false) &&
 		(stemLanguage.empty() == false))
 	{
-#ifdef DEBUG
-		cout << "XapianEngine::parseQuery: " << stemLanguage << " stemming" << endl;
-#endif
-		try
-		{
-			stemmer = Xapian::Stem(StringManip::toLowerCase(stemLanguage));
-		}
-		catch (const Xapian::Error &error)
-		{
-			cerr << "Couldn't create stemmer: " << error.get_type() << ": " << error.get_msg() << endl;
-		}
-		parser.set_stemmer(stemmer);
+		parser.set_stemmer(m_stemmer);
 		parser.set_stemming_strategy(Xapian::QueryParser::STEM_SOME);
 
 		// Don't bother loading the stopwords list if there's only one token
 		if (tokensCount > 1)
 		{
-			string languageCode(Languages::toCode(stemLanguage));
-
-			FileStopper *pStopper = FileStopper::get_stopper(languageCode);
+			FileStopper *pStopper = FileStopper::get_stopper(Languages::toCode(stemLanguage));
 			if ((pStopper != NULL) &&
 				(pStopper->get_stopwords_count() > 0))
 			{
@@ -586,6 +682,11 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 	{
 		parser.set_default_op(Xapian::Query::OP_OR);
 	}
+#if XAPIAN_NUM_VERSION >= 1000004
+	// Search across text body and title
+	parser.add_prefix("", "");
+	parser.add_prefix("", "S");
+#endif
 	// X prefixes should always include a colon
 	parser.add_boolean_prefix("site", "H");
 	parser.add_boolean_prefix("file", "P");
@@ -593,6 +694,8 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 	parser.add_prefix("title", "S");
 	parser.add_boolean_prefix("url", "U");
 	parser.add_boolean_prefix("dir", "XDIR:");
+	parser.add_boolean_prefix("inurl", "XFILE:");
+	parser.add_prefix("path", "XPATH:");
 	parser.add_boolean_prefix("lang", "L");
 	parser.add_boolean_prefix("type", "T");
 	parser.add_boolean_prefix("class", "XCLASS:");
@@ -742,7 +845,6 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 
 	if (minimal == false)
 	{
-
 #if ENABLE_XAPIAN_SPELLING_CORRECTION>0
 		// Any correction ?
 		correctedFreeQuery = parser.get_corrected_query_string();
@@ -759,7 +861,7 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 }
 
 bool XapianEngine::queryDatabase(Xapian::Database *pIndex, Xapian::Query &query,
-	unsigned int startDoc, const QueryProperties &queryProps)
+	const string &stemLanguage, unsigned int startDoc, const QueryProperties &queryProps)
 {
 	Timer timer;
 	unsigned int maxResultsCount = queryProps.getMaximumResultsCount();
@@ -801,9 +903,9 @@ bool XapianEngine::queryDatabase(Xapian::Database *pIndex, Xapian::Query &query,
 
 		// Get the top results of the query
 		Xapian::MSet matches = enquire.get_mset(startDoc, maxResultsCount, (2 * maxResultsCount) + 1);
+		m_resultsCountEstimate = matches.get_matches_estimated();
 		if (matches.empty() == false)
 		{
-			m_resultsCountEstimate = matches.get_matches_estimated();
 #ifdef DEBUG
 			cout << "XapianEngine::queryDatabase: found " << matches.size() << "/" << maxResultsCount
 				<< " results found from position " << startDoc << endl;
@@ -891,7 +993,7 @@ bool XapianEngine::queryDatabase(Xapian::Database *pIndex, Xapian::Query &query,
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "XapianEngine::queryDatabase: " << error.get_type() << ": " << error.get_msg() << endl;
+		cerr << "Couldn't run query: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	cout << "Ran query \"" << queryProps.getFreeQuery() << "\" in " << timer.stop() << " ms" << endl;
 
@@ -903,7 +1005,6 @@ bool XapianEngine::queryDatabase(Xapian::Database *pIndex, Xapian::Query &query,
 		if (m_expandDocuments.empty() == false)
 		{
 			Xapian::RSet expandDocs;
-			unsigned int count = 0;
 
 			for (set<string>::const_iterator docIter = m_expandDocuments.begin();
 				docIter != m_expandDocuments.end(); ++docIter)
@@ -923,25 +1024,19 @@ bool XapianEngine::queryDatabase(Xapian::Database *pIndex, Xapian::Query &query,
 
 			// Get 10 non-prefixed terms
 			string allowedPrefixes("RS");
-			PrefixDecider expandDecider(allowedPrefixes);
-			CJKVTokenizer tokenizer;
-			Xapian::ESet expandTerms = enquire.get_eset(20, expandDocs, &expandDecider);
+			TermDecider expandDecider(pIndex, ((stemLanguage.empty() == true) ? NULL : &m_stemmer),
+				FileStopper::get_stopper(Languages::toCode(stemLanguage)),
+				allowedPrefixes, query);
+			Xapian::ESet expandTerms = enquire.get_eset(10, expandDocs, &expandDecider);
 #ifdef DEBUG
 			cout << "XapianEngine::queryDatabase: " << expandTerms.size() << " expand terms" << endl;
 #endif
 			for (Xapian::ESetIterator termIter = expandTerms.begin();
-				(termIter != expandTerms.end()) && (count < 10); ++termIter)
+				termIter != expandTerms.end(); ++termIter)
 			{
 				string expandTerm(*termIter);
 				char firstChar = expandTerm[0];
 
-				// Skip short terms
-				if ((tokenizer.has_cjkv(expandTerm) == false) &&
-					(expandTerm.length() < 3))
-				{
-					continue;
-				}
-
 				// Is this prefixed ?
 				if (allowedPrefixes.find(firstChar) != string::npos)
 				{
@@ -949,13 +1044,12 @@ bool XapianEngine::queryDatabase(Xapian::Database *pIndex, Xapian::Query &query,
 				}
 
 				m_expandTerms.insert(expandTerm);
-				++count;
 			}
 		}
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "XapianEngine::queryDatabase: " << error.get_type() << ": " << error.get_msg() << endl;
+		cerr << "Couldn't run query: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 
 	// Be tolerant of errors as long as we got some results
@@ -1035,6 +1129,8 @@ bool XapianEngine::setExpandSet(const set<string> &docsSet)
 bool XapianEngine::runQuery(QueryProperties& queryProps,
 	unsigned int startDoc)
 {
+	string stemLanguage(Languages::toEnglish(queryProps.getStemmingLanguage()));
+
 	// Clear the results list
 	m_resultsList.clear();
 	m_resultsCountEstimate = 0;
@@ -1051,26 +1147,41 @@ bool XapianEngine::runQuery(QueryProperties& queryProps,
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, true);
 	if (pDatabase == NULL)
 	{
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
+	if (stemLanguage.empty() == false)
+	{
+#ifdef DEBUG
+		cout << "XapianEngine::runQuery: " << stemLanguage << " stemming" << endl;
+#endif
+		try
+		{
+			m_stemmer = Xapian::Stem(StringManip::toLowerCase(stemLanguage));
+		}
+		catch (const Xapian::Error &error)
+		{
+			cerr << "Couldn't create stemmer: " << error.get_type() << ": " << error.get_msg() << endl;
+		}
+	}
+
 	// Get the latest revision...
 	pDatabase->reopen();
 	Xapian::Database *pIndex = pDatabase->readLock();
 	try
 	{
-		string stemLanguage(Languages::toEnglish(queryProps.getStemmingLanguage()));
 		unsigned int searchStep = 1;
 
 		// Searches are run in this order :
-		// 1. don't stem terms
-		// 2. if no results, stem terms if a language is defined for the query
+		// 1. no stemming, exact matches only
+		// 2. stem terms if a language is defined for the query
 		Xapian::Query fullQuery = parseQuery(pIndex, queryProps, "",
 			m_defaultOperator, m_limitQuery, m_correctedFreeQuery);
 		while (fullQuery.empty() == false)
 		{
 			// Query the database
-			if (queryDatabase(pIndex, fullQuery, startDoc, queryProps) == false)
+			if (queryDatabase(pIndex, fullQuery, stemLanguage, startDoc, queryProps) == false)
 			{
 				break;
 			}
@@ -1102,7 +1213,7 @@ bool XapianEngine::runQuery(QueryProperties& queryProps,
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "XapianEngine::runQuery: " << error.get_type() << ": " << error.get_msg() << endl;
+		cerr << "Couldn't run query: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	pDatabase->unlock();
 
