@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005-2008 Fabrice Colin
+ *  Copyright 2005-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <strings.h>
 #include <time.h>
 #include <ctype.h>
+#include <vector>
 #include <iostream>
 #include <fstream>
 #include <algorithm>
@@ -55,6 +56,18 @@ using std::map;
 using std::min;
 using std::max;
 
+static string removeCharsetFromType(const string &type)
+{
+	// Remove the charset, if any
+	string::size_type semiColonPos = type.find(";");
+	if (semiColonPos != string::npos)
+	{
+		return type.substr(0, semiColonPos);
+	}
+
+	return type;
+}
+
 class TokensIndexer : public Dijon::CJKVTokenizer::TokensHandler
 {
 	public:
@@ -235,7 +248,7 @@ bool XapianIndex::listDocumentsWithTerm(const string &term, set<unsigned int> &d
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return 0;
 	}
 
@@ -532,40 +545,37 @@ void XapianIndex::removePostingsFromDocument(const Xapian::Utf8Iterator &itor, X
 void XapianIndex::addCommonTerms(const DocumentInfo &info, Xapian::Document &doc,
 	const Xapian::WritableDatabase &db, Xapian::termcount &termPos)
 {
+	Dijon::CJKVTokenizer pathTokenizer;
+	vector<string> paths;
 	string title(info.getTitle());
 	string location(info.getLocation());
-	string type(info.getType());
+	string type(removeCharsetFromType(info.getType()));
 	Url urlObj(location);
 
 	// Add a magic term :-)
 	doc.add_term(MAGIC_TERM);
 
-	// Index the title with and without prefix S
+	// Index the title with prefix S
 	if (title.empty() == false)
 	{
 		addPostingsToDocument(Xapian::Utf8Iterator(title), doc, db, "S",
 			true, m_doSpelling, termPos);
-		addPostingsToDocument(Xapian::Utf8Iterator(title), doc, db, "",
-			false, m_doSpelling, termPos);
-
-		// Make room between common terms and what follows
-		termPos += 100;
 	}
 
 	// Index the full URL with prefix U
-	doc.add_term(string("U") + XapianDatabase::limitTermLength(Url::escapeUrl(location), true));
+	doc.add_term(string("U") + XapianDatabase::limitTermLength(Url::escapeUrl(info.getLocation(true)), true));
 	// ...the base file with XFILE:
-	string::size_type qmPos = location.find("?");
 	if ((urlObj.isLocal() == true) &&
-		(qmPos != string::npos))
+		(info.getInternalPath().empty() == false))
 	{
-		string fileUrl(location.substr(0, qmPos));
 		string protocol(urlObj.getProtocol());
 
-		doc.add_term(string("XFILE:") + XapianDatabase::limitTermLength(Url::escapeUrl(fileUrl), true));
+		doc.add_term(string("XFILE:") + XapianDatabase::limitTermLength(Url::escapeUrl(location), true));
 		if ((urlObj.isLocal() == true) &&
 			(protocol != "file"))
 		{
+			string fileUrl(location);
+
 			// Add another term with file as protocol
 			fileUrl.replace(0, protocol.length(), "file");
 			doc.add_term(string("XFILE:") + XapianDatabase::limitTermLength(Url::escapeUrl(fileUrl), true));
@@ -602,6 +612,18 @@ void XapianIndex::addCommonTerms(const DocumentInfo &info, Xapian::Document &doc
 			// Next
 			slashPos = tree.find('/', slashPos + 1);
 		}
+
+		// ...and all components as XPATH:
+		pathTokenizer.tokenize(tree, paths);
+		for (vector<string>::iterator pathIter = paths.begin();
+			pathIter != paths.end(); ++pathIter)
+		{
+			doc.add_term(string("XPATH:") + XapianDatabase::limitTermLength(Url::escapeUrl(StringManip::toLowerCase(*pathIter)), true));
+		}
+	}
+	else
+	{
+		doc.add_term("XDIR:/");
 	}
 	// ...and the file name with prefix P
 	string fileName(urlObj.getFile());
@@ -610,6 +632,17 @@ void XapianIndex::addCommonTerms(const DocumentInfo &info, Xapian::Document &doc
 		string extension;
 
 		doc.add_term(string("P") + XapianDatabase::limitTermLength(Url::escapeUrl(fileName), true));
+		if (fileName.find(' ') != string::npos)
+		{
+			// Add more XPATH: terms if there's a space in the file name
+			paths.clear();
+			pathTokenizer.tokenize(fileName, paths);
+			for (vector<string>::iterator pathIter = paths.begin();
+				pathIter != paths.end(); ++pathIter)
+			{
+				doc.add_term(string("XPATH:") + XapianDatabase::limitTermLength(Url::escapeUrl(StringManip::toLowerCase(*pathIter)), true));
+			}
+		}
 
 		// Does it have an extension ?
 		string::size_type extPos = fileName.rfind('.');
@@ -634,7 +667,9 @@ void XapianIndex::addCommonTerms(const DocumentInfo &info, Xapian::Document &doc
 void XapianIndex::removeCommonTerms(Xapian::Document &doc, const Xapian::WritableDatabase &db)
 {
 	DocumentInfo docInfo;
+	Dijon::CJKVTokenizer pathTokenizer;
 	set<string> commonTerms;
+	vector<string> paths;
 	string record(doc.get_data());
 
 	// First, remove the magic term
@@ -646,23 +681,9 @@ void XapianIndex::removeCommonTerms(Xapian::Document &doc, const Xapian::Writabl
 		return;
 	}
 
-	string language(StringManip::extractField(record, "language=", "\n"));
-
-	docInfo = DocumentInfo(StringManip::extractField(record, "caption=", "\n"),
-		StringManip::extractField(record, "url=", "\n"),
-		StringManip::extractField(record, "type=", "\n"),
-		Languages::toLocale(language));
-	string modTime(StringManip::extractField(record, "modtime=", "\n"));
-	if (modTime.empty() == false)
-	{
-		time_t timeT = (time_t )atol(modTime.c_str());
-		docInfo.setTimestamp(TimeConverter::toTimestamp(timeT));
-	}
-	string bytesSize(StringManip::extractField(record, "size=", ""));
-	if (bytesSize.empty() == false)
-	{
-		docInfo.setSize((off_t )atol(bytesSize.c_str()));
-	}
+	XapianDatabase::recordToProps(record, &docInfo);
+	// XapianDatabase expects the language in English, which is okay here
+	string language(docInfo.getLanguage());
 	Url urlObj(docInfo.getLocation());
 
 	// FIXME: remove terms extracted from the title if they don't have more than one posting
@@ -671,26 +692,24 @@ void XapianIndex::removeCommonTerms(Xapian::Document &doc, const Xapian::Writabl
 	{
 		removePostingsFromDocument(Xapian::Utf8Iterator(title), doc, db, "S",
 			language, true, m_doSpelling);
-		removePostingsFromDocument(Xapian::Utf8Iterator(title), doc, db, "",
-			language, false, m_doSpelling);
 	}
 
 	// Location 
 	string location(docInfo.getLocation());
-	commonTerms.insert(string("U") + XapianDatabase::limitTermLength(Url::escapeUrl(location), true));
+	commonTerms.insert(string("U") + XapianDatabase::limitTermLength(Url::escapeUrl(docInfo.getLocation(true)), true));
 	// Base file
-	string::size_type qmPos = location.find("?");
 	if ((urlObj.isLocal() == true) &&
-		(qmPos != string::npos))
+		(docInfo.getInternalPath().empty() == false))
 	{
-		string fileUrl(location.substr(0, qmPos));
 		string protocol(urlObj.getProtocol());
 
-		commonTerms.insert(string("XFILE:") + XapianDatabase::limitTermLength(Url::escapeUrl(fileUrl), true));
+		commonTerms.insert(string("XFILE:") + XapianDatabase::limitTermLength(Url::escapeUrl(location), true));
 
 		if ((urlObj.isLocal() == true) &&
 			(protocol != "file"))
 		{
+			string fileUrl(location);
+
 			// Add another term with file as protocol
 			fileUrl.replace(0, protocol.length(), "file");
 			doc.add_term(string("XFILE:") + XapianDatabase::limitTermLength(Url::escapeUrl(fileUrl), true));
@@ -727,6 +746,18 @@ void XapianIndex::removeCommonTerms(Xapian::Document &doc, const Xapian::Writabl
 			// Next
 			slashPos = tree.find('/', slashPos + 1);
 		}
+
+		// ...paths
+		pathTokenizer.tokenize(tree, paths);
+		for (vector<string>::iterator pathIter = paths.begin();
+			pathIter != paths.end(); ++pathIter)
+		{
+			commonTerms.insert(string("XPATH:") + XapianDatabase::limitTermLength(Url::escapeUrl(StringManip::toLowerCase(*pathIter)), true));
+		}
+	}
+	else
+	{
+		commonTerms.insert("XDIR:/");
 	}
 	// ...and file name
 	string fileName(urlObj.getFile());
@@ -735,6 +766,16 @@ void XapianIndex::removeCommonTerms(Xapian::Document &doc, const Xapian::Writabl
 		string extension;
 
 		commonTerms.insert(string("P") + XapianDatabase::limitTermLength(Url::escapeUrl(fileName), true));
+		if (fileName.find(' ') != string::npos)
+		{
+			paths.clear();
+			pathTokenizer.tokenize(fileName, paths);
+			for (vector<string>::iterator pathIter = paths.begin();
+				pathIter != paths.end(); ++pathIter)
+			{
+				commonTerms.insert(string("XPATH:") + XapianDatabase::limitTermLength(Url::escapeUrl(StringManip::toLowerCase(*pathIter)), true));
+			}
+		}
 
 		// Does it have an extension ?
 		string::size_type extPos = fileName.rfind('.');
@@ -748,7 +789,7 @@ void XapianIndex::removeCommonTerms(Xapian::Document &doc, const Xapian::Writabl
 	// Language code
 	commonTerms.insert(string("L") + Languages::toCode(language));
 	// MIME type
-	string type(docInfo.getType());
+	string type(removeCharsetFromType(docInfo.getType()));
 	commonTerms.insert(string("T") + type);
 	string::size_type slashPos = type.find('/');
 	if (slashPos != string::npos)
@@ -774,7 +815,6 @@ void XapianIndex::removeCommonTerms(Xapian::Document &doc, const Xapian::Writabl
 string XapianIndex::scanDocument(const string &suggestedLanguage,
 	const char *pData, unsigned int dataLength)
 {
-	LanguageDetector lang;
 	vector<string> candidates;
 	string language;
 	bool scannedDocument = false;
@@ -787,7 +827,7 @@ string XapianIndex::scanDocument(const string &suggestedLanguage,
 	else
 	{
 		// Try to determine the document's language right away
-		lang.guessLanguage(pData, max(dataLength, (unsigned int)2048), candidates);
+		LanguageDetector::getInstance().guessLanguage(pData, max(dataLength, (unsigned int)2048), candidates);
 
 		scannedDocument = true;
 	}
@@ -804,7 +844,7 @@ string XapianIndex::scanDocument(const string &suggestedLanguage,
 
 		try
 		{
-			Xapian::Stem stemmer(*langIter);
+			Xapian::Stem stemmer(StringManip::toLowerCase(*langIter));
 		}
 		catch (const Xapian::Error &error)
 		{
@@ -814,7 +854,7 @@ string XapianIndex::scanDocument(const string &suggestedLanguage,
 			{
 				// The suggested language is not suitable
 				candidates.clear();
-				lang.guessLanguage(pData, max(dataLength, (unsigned int)2048), candidates);
+				LanguageDetector::getInstance().guessLanguage(pData, max(dataLength, (unsigned int)2048), candidates);
 
 				langIter = candidates.begin();
 				scannedDocument = true;
@@ -872,7 +912,7 @@ bool XapianIndex::deleteDocuments(const string &term)
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -923,7 +963,7 @@ string XapianIndex::getMetadata(const string &name) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return "";
 	}
 
@@ -962,7 +1002,7 @@ bool XapianIndex::setMetadata(const string &name, const string &value) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1014,7 +1054,7 @@ bool XapianIndex::getDocumentInfo(unsigned int docId, DocumentInfo &docInfo) con
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1057,7 +1097,7 @@ unsigned int XapianIndex::getDocumentTermsCount(unsigned int docId) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return 0;
 	}
 
@@ -1096,7 +1136,7 @@ bool XapianIndex::getDocumentTerms(unsigned int docId, map<unsigned int, string>
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1173,10 +1213,11 @@ bool XapianIndex::getDocumentTerms(unsigned int docId, map<unsigned int, string>
 }
 
 /// Sets the list of known labels.
-bool XapianIndex::setLabels(const set<string> &labels)
+bool XapianIndex::setLabels(const set<string> &labels, bool resetLabels)
 {
 	string labelsString;
 
+	// Whether labels are reset or not doesn't make any difference
 	for (set<string>::const_iterator labelIter = labels.begin();
 		labelIter != labels.end(); ++labelIter)
 	{
@@ -1228,65 +1269,6 @@ bool XapianIndex::addLabel(const string &name)
 	return false;
 }
 
-/// Renames a label.
-bool XapianIndex::renameLabel(const string &name, const string &newName)
-{
-	bool renamedLabel = false;
-
-	// Prevent from renaming or setting internal labels
-	if ((name.substr(0, 2) == "X-") ||
-		(newName.substr(0, 2) == "X-"))
-	{
-		return false;
-	}
-
-	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
-	if (pDatabase == NULL)
-	{
-		cerr << "Bad index " << m_databaseName << endl;
-		return false;
-	}
-
-	try
-	{
-		Xapian::WritableDatabase *pIndex = pDatabase->writeLock();
-		if (pIndex != NULL)
-		{
-			string term("XLABEL:");
-
-			// Get documents that have this label
-			term += XapianDatabase::limitTermLength(Url::escapeUrl(name));
-			for (Xapian::PostingIterator postingIter = pIndex->postlist_begin(term);
-				postingIter != pIndex->postlist_end(term); ++postingIter)
-			{
-				Xapian::docid docId = *postingIter;
-
-				// Get the document
-				Xapian::Document doc = pIndex->get_document(docId);
-				// Remove the term
-				doc.remove_term(term);
-				// ...add the new one
-				doc.add_term(string("XLABEL:") + XapianDatabase::limitTermLength(Url::escapeUrl(newName)));
-				// ...and update the document
-				pIndex->replace_document(docId, doc);
-			}
-
-			renamedLabel = true;
-		}
-	}
-	catch (const Xapian::Error &error)
-	{
-		cerr << "Couldn't delete label: " << error.get_type() << ": " << error.get_msg() << endl;
-	}
-	catch (...)
-	{
-		cerr << "Couldn't delete label, unknown exception occured" << endl;
-	}
-	pDatabase->unlock();
-
-	return renamedLabel;
-}
-
 /// Deletes all references to a label.
 bool XapianIndex::deleteLabel(const string &name)
 {
@@ -1301,7 +1283,7 @@ bool XapianIndex::deleteLabel(const string &name)
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1350,7 +1332,7 @@ bool XapianIndex::hasLabel(unsigned int docId, const string &name) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1398,7 +1380,7 @@ bool XapianIndex::getDocumentLabels(unsigned int docId, set<string> &labels) con
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1461,7 +1443,7 @@ bool XapianIndex::setDocumentsLabels(const set<unsigned int> &docIds,
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1529,7 +1511,7 @@ unsigned int XapianIndex::hasDocument(const string &url) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return 0;
 	}
 
@@ -1581,7 +1563,7 @@ unsigned int XapianIndex::getCloseTerms(const string &term, set<string> &suggest
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return 0;
 	}
 
@@ -1637,7 +1619,7 @@ unsigned int XapianIndex::getLastDocumentID(void) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return 0;
 	}
 
@@ -1670,7 +1652,7 @@ unsigned int XapianIndex::getDocumentsCount(const string &labelName) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return 0;
 	}
 
@@ -1746,16 +1728,13 @@ bool XapianIndex::indexDocument(const Document &document, const std::set<std::st
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
 	// Cache the document's properties
-	DocumentInfo docInfo(document.getTitle(), document.getLocation(),
-		document.getType(), document.getLanguage());
-	docInfo.setTimestamp(document.getTimestamp());
-	docInfo.setSize(document.getSize());
-	docInfo.setLocation(Url::canonicalizeUrl(docInfo.getLocation()));
+	DocumentInfo docInfo(document);
+	docInfo.setLocation(Url::canonicalizeUrl(document.getLocation()));
 
 	unsigned int dataLength = 0;
 	const char *pData = document.getData(dataLength);
@@ -1787,7 +1766,7 @@ bool XapianIndex::indexDocument(const Document &document, const std::set<std::st
 					false, m_doSpelling, termPos);
 			}
 #ifdef DEBUG
-			cout << "XapianIndex::indexDocument: " << labels.size() << " labels for URL " << docInfo.getLocation() << endl;
+			cout << "XapianIndex::indexDocument: " << labels.size() << " labels for URL " << docInfo.getLocation(true) << endl;
 #endif
 
 			// Add labels
@@ -1822,16 +1801,13 @@ bool XapianIndex::updateDocument(unsigned int docId, const Document &document)
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
 	// Cache the document's properties
-	DocumentInfo docInfo(document.getTitle(), document.getLocation(),
-		document.getType(), document.getLanguage());
-	docInfo.setTimestamp(document.getTimestamp());
-	docInfo.setSize(document.getSize());
-	docInfo.setLocation(Url::canonicalizeUrl(docInfo.getLocation()));
+	DocumentInfo docInfo(document);
+	docInfo.setLocation(Url::canonicalizeUrl(document.getLocation()));
 
 	unsigned int dataLength = 0;
 	const char *pData = document.getData(dataLength);
@@ -1910,7 +1886,7 @@ bool XapianIndex::updateDocumentInfo(unsigned int docId, const DocumentInfo &doc
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -1958,7 +1934,7 @@ bool XapianIndex::unindexDocument(unsigned int docId)
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -2029,7 +2005,7 @@ bool XapianIndex::flush(void)
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
@@ -2062,9 +2038,10 @@ bool XapianIndex::reopen(void) const
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
+	pDatabase->reopen();
 
 	return true;
 }
@@ -2076,7 +2053,7 @@ bool XapianIndex::reset(void)
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false, true);
 	if (pDatabase == NULL)
 	{
-		cerr << "Bad index " << m_databaseName << endl;
+		cerr << "Couldn't get index " << m_databaseName << endl;
 		return false;
 	}
 
