@@ -1,6 +1,6 @@
 /*
- *  Copyright 2007,2008 林永忠 Yung-Chung Lin
- *  Copyright 2008 Fabrice Colin
+ *  Copyright 2007-2008 林永忠 Yung-Chung Lin
+ *  Copyright 2008-2009 Fabrice Colin
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -17,8 +17,9 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <cctype>
-#include <cstring>
+#include <ctype.h>
+#include <string.h>
+#include <iostream>
 
 #include "CJKVTokenizer.h"
 
@@ -38,6 +39,26 @@ static int unicode_strlen(const char *p, int max)
 {
 	return (int)g_utf8_strlen(p, (gssize)max);
 }
+
+static int unicode_ispunct(unicode_char_t c)
+{
+	if (g_unichar_ispunct(c))
+	{
+		return 1;
+	}
+
+	return 0;
+}
+
+static int unicode_isspace(unicode_char_t c)
+{
+	if (g_unichar_isspace(c))
+	{
+		return 1;
+	}
+
+	return 0;
+}
 #endif
 
 // 2E80..2EFF; CJK Radicals Supplement
@@ -120,7 +141,12 @@ static inline unsigned char *_unicode_to_char(unicode_char_t &uchar,
 	}
 
 	memset(p, 0, sizeof(unicode_char_t) + 1);
-	if (uchar < 0x80)
+	if (unicode_isspace(uchar) ||
+		unicode_ispunct(uchar))
+	{
+		p[0] = ' ';
+	}
+	else if (uchar < 0x80)
 	{
 		p[0] = uchar;
 	}
@@ -172,7 +198,8 @@ class VectorTokensHandler : public CJKVTokenizer::TokensHandler
 
 CJKVTokenizer::CJKVTokenizer() :
 	m_nGramSize(2),
-	m_maxTokenCount(0)
+	m_maxTokenCount(0),
+	m_maxTextSize(5242880)
 {
 	unicode_init();
 }
@@ -201,6 +228,16 @@ unsigned int CJKVTokenizer::get_max_token_count(void) const
 	return m_maxTokenCount;
 }
 
+void CJKVTokenizer::set_max_text_size(unsigned int max_text_size)
+{
+	m_maxTextSize = max_text_size;
+}
+
+unsigned int CJKVTokenizer::get_max_text_size(void) const
+{
+	return m_maxTextSize;
+}
+
 void CJKVTokenizer::tokenize(const string &str, vector<string> &token_list)
 {
 	VectorTokensHandler handler(token_list);
@@ -226,7 +263,7 @@ void CJKVTokenizer::tokenize(const string &str, TokensHandler &handler,
 		{
 			break;
 		}
-		token_str.clear();
+		token_str.resize(0);
 		if (UTF8_IS_CJKV(temp_uchar_list[i]))
 		{
 			for (unsigned int j = i; j < i + m_nGramSize; j++)
@@ -254,6 +291,7 @@ void CJKVTokenizer::tokenize(const string &str, TokensHandler &handler,
 		else
 		{
 			unsigned int j = i;
+
 			while (j < temp_token_list.size())
 			{
 				unsigned char *p = (unsigned char*) temp_token_list[j].c_str();
@@ -283,6 +321,7 @@ void CJKVTokenizer::tokenize(const string &str, TokensHandler &handler,
 				{
 					break;
 				}
+
 				token_str += temp_token_list[j];
 				j++;
 			}
@@ -292,7 +331,7 @@ void CJKVTokenizer::tokenize(const string &str, TokensHandler &handler,
 			{
 				break;
 			}
-			if(token_str.length() > 0)
+			if (token_str.length() > 0)
 			{
 				if (handler.handle_token(token_str, false) == true)
 				{
@@ -306,35 +345,46 @@ void CJKVTokenizer::tokenize(const string &str, TokensHandler &handler,
 void CJKVTokenizer::split(const string &str, vector<string> &token_list)
 {
 	unicode_char_t uchar;
-	char *str_ptr = (char*) str.c_str();
+	const char *str_ptr = str.c_str();
 	int str_utf8_len = unicode_strlen(str_ptr, str.length());
 	unsigned char p[sizeof(unicode_char_t) + 1];
 
 	for (int i = 0; i < str_utf8_len; i++)
 	{
-		str_ptr = unicode_get_utf8((const char*) str_ptr, &uchar);
+		str_ptr = unicode_get_utf8(str_ptr, &uchar);
 		if (str_ptr == NULL)
 		{
 			break;
 		}
 
-		token_list.push_back(string((const char*)_unicode_to_char(uchar, p)));
+		if (i >= m_maxTextSize)
+		{
+			break;
+		}
+
+		token_list.push_back((const char*)_unicode_to_char(uchar, p));
 	}
 }
 
 void CJKVTokenizer::split(const string &str, vector<unicode_char_t> &token_list)
 {
 	unicode_char_t uchar;
-	char *str_ptr = (char*) str.c_str();
+	const char *str_ptr = str.c_str();
 	int str_utf8_len = unicode_strlen(str_ptr, str.length());
 
 	for (int i = 0; i < str_utf8_len; i++)
 	{
-		str_ptr = unicode_get_utf8((const char*) str_ptr, &uchar);
+		str_ptr = unicode_get_utf8(str_ptr, &uchar);
 		if (str_ptr == NULL)
 		{
 			break;
 		}
+
+		if (i >= m_maxTextSize)
+		{
+			break;
+		}
+
 		token_list.push_back(uchar);
 	}
 }
