@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005-2008 Fabrice Colin
+ *  Copyright 2005-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -49,15 +49,21 @@ string CrawlHistory::statusToText(CrawlStatus status)
 		case UNKNOWN:
 			text = "UNKNOWN";
 			break;
+		case TO_CRAWL:
+			text = "TO_CRAWL";
+			break;
 		case CRAWLING:
 			text = "CRAWLING";
 			break;
 		case CRAWLED:
 			text = "CRAWLED";
 			break;
-		case ERROR:
+		case CRAWL_ERROR:
 			text = "ERROR";
 			break;
+		case CRAWL_LINK:
+			text = "LINK";
+			break;
 		default:
 			break;
 	}
@@ -69,7 +75,11 @@ CrawlHistory::CrawlStatus CrawlHistory::textToStatus(const string &text)
 {
 	CrawlStatus status = UNKNOWN;
 
-	if (text == "CRAWLING")
+	if (text == "TO_CRAWL")
+	{
+		status = TO_CRAWL;
+	}
+	else if (text == "CRAWLING")
 	{
 		status = CRAWLING;
 	}
@@ -79,7 +89,11 @@ CrawlHistory::CrawlStatus CrawlHistory::textToStatus(const string &text)
 	}
 	else if (text == "ERROR")
 	{
-		status = ERROR;
+		status = CRAWL_ERROR;
+	}
+	else if (text == "LINK")
+	{
+		status = CRAWL_LINK;
 	}
 
 	return status;
@@ -132,7 +146,7 @@ bool CrawlHistory::create(const string &database)
 	if (createHistoryTable == true)
 	{
 		if (db.executeSimpleStatement("CREATE TABLE CrawlHistory (Url VARCHAR(255) PRIMARY KEY, \
-			Status VARCHAR(255), SourceID INTEGER, DATE INTEGER, ErrorNum INTEGER);") == false)
+			Status VARCHAR(255), SourceID INTEGER, Date INTEGER, ErrorNum INTEGER);") == false)
 		{
 			return false;
 		}
@@ -326,7 +340,7 @@ bool CrawlHistory::updateItems(const map<string, time_t> urls, CrawlStatus statu
 }
 
 /// Updates the status of items en masse.
-bool CrawlHistory::updateItemsStatus(CrawlStatus currentStatus, CrawlStatus newStatus,
+bool CrawlHistory::updateItemsStatus(CrawlStatus oldStatus, CrawlStatus newStatus,
 	unsigned int sourceId, bool allSources)
 {
 	SQLResults *results = NULL;
@@ -336,16 +350,14 @@ bool CrawlHistory::updateItemsStatus(CrawlStatus currentStatus, CrawlStatus newS
 	{
 		results = executeStatement("UPDATE CrawlHistory \
 			SET Status='%q' WHERE SourceId='%u' AND Status='%q';",
-			statusToText(newStatus).c_str(), sourceId,
-			statusToText(currentStatus).c_str());
+			statusToText(newStatus).c_str(), sourceId, statusToText(oldStatus).c_str());
 	}
 	else
 	{
 		// Ignore the source
 		results = executeStatement("UPDATE CrawlHistory \
 			SET Status='%q' WHERE Status='%q';",
-			statusToText(newStatus).c_str(),
-			statusToText(currentStatus).c_str());
+			statusToText(newStatus).c_str(), statusToText(oldStatus).c_str());
 	}
 
 	if (results != NULL)
@@ -412,7 +424,8 @@ unsigned int CrawlHistory::getItems(CrawlStatus status, set<string> &urls)
 
 /// Returns items that belong to a source.
 unsigned int CrawlHistory::getSourceItems(unsigned int sourceId, CrawlStatus status,
-	set<string> &urls, time_t minDate)
+	set<string> &urls, unsigned int min, unsigned int max,
+	time_t minDate)
 {
 	SQLResults *results = NULL;
 	unsigned int count = 0;
@@ -420,15 +433,15 @@ unsigned int CrawlHistory::getSourceItems(unsigned int sourceId, CrawlStatus sta
 	if (minDate > 0)
 	{
 		results = executeStatement("SELECT Url FROM CrawlHistory \
-			WHERE SourceId='%u' AND Status='%q' AND Date>'%d';",
-			sourceId, statusToText(status).c_str(), minDate);
+			WHERE SourceId='%u' AND Status='%q' AND Date>'%d' LIMIT %u OFFSET %u;",
+			sourceId, statusToText(status).c_str(), minDate, max - min, min);
 	}
 	else
 	{
 		// Ignore the date
 		results = executeStatement("SELECT Url FROM CrawlHistory \
-			WHERE SourceId='%u' AND Status='%q';",
-			sourceId, statusToText(status).c_str());
+			WHERE SourceId='%u' AND Status='%q' LIMIT %u OFFSET %u;",
+			sourceId, statusToText(status).c_str(), max - min, min);
 	}
 
 	if (results != NULL)
