@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005-2008 Fabrice Colin
+ *  Copyright 2005-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@ class CrawlHistory : public SQLiteBase
 		CrawlHistory(const std::string &database);
 		virtual ~CrawlHistory();
 
-		typedef enum { UNKNOWN, CRAWLING, CRAWLED, ERROR } CrawlStatus;
+		typedef enum { UNKNOWN, TO_CRAWL, CRAWLING, CRAWLED, CRAWL_ERROR, CRAWL_LINK } CrawlStatus;
 
 		/// Creates the CrawlHistory table in the database.
 		static bool create(const std::string &database);
@@ -64,7 +64,7 @@ class CrawlHistory : public SQLiteBase
 		bool updateItems(const std::map<std::string, time_t> urls, CrawlStatus status);
 
 		/// Updates the status of items en masse.
-		bool updateItemsStatus(CrawlStatus currentStatus, CrawlStatus newStatus,
+		bool updateItemsStatus(CrawlStatus oldStatus, CrawlStatus newStatus,
 			unsigned int sourceId, bool allSources = false);
 
 		/// Gets the error number and date for a URL.
@@ -75,7 +75,9 @@ class CrawlHistory : public SQLiteBase
 
 		/// Returns items that belong to a source.
 		unsigned int getSourceItems(unsigned int sourceId, CrawlStatus status,
-			std::set<std::string> &urls, time_t minDate = 0);
+			std::set<std::string> &urls,
+			unsigned int min, unsigned int max,
+			time_t minDate = 0);
 
 		/// Returns the number of URLs.
 		unsigned int getItemsCount(CrawlStatus status);
