@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -23,6 +23,7 @@
 #include <utility>
 #include <glibmm/convert.h>
 #include <gtkmm/stock.h>
+#include <gtkmm/targetlist.h>
 
 #include "config.h"
 #include "DocumentInfo.h"
@@ -36,6 +37,7 @@
 
 using namespace std;
 using namespace Glib;
+using namespace Gdk;
 using namespace Gtk;
 
 importDialog::importDialog() :
@@ -51,6 +53,13 @@ importDialog::importDialog() :
 	m_refLocationCompletion->set_popup_completion(true);
 	locationEntry->set_completion(m_refLocationCompletion);
 
+	// Specify drop targets
+	locationEntry->drag_dest_set(Gtk::DEST_DEFAULT_ALL, DragAction(GDK_ACTION_COPY|GDK_ACTION_MOVE));
+	locationEntry->drag_dest_add_uri_targets();
+	// Connect to the drag data received signal
+	locationEntry->signal_drag_data_received().connect(
+		sigc::mem_fun(*this, &importDialog::on_data_received));
+
 	// Populate
 	populate_comboboxes();
 
@@ -68,6 +77,25 @@ const DocumentInfo &importDialog::getDocumentInfo(void) const
 	return m_docInfo;
 }
 
+void importDialog::on_data_received(const RefPtr<DragContext> &context,
+	int x, int y, const SelectionData &data, guint info, guint time)
+{
+	list<ustring> droppedUris = data.get_uris();
+	bool goodDrop = false;
+
+	cout << "importDialog::on_data_received: data type "
+		<< data.get_data_type() << " in format " << data.get_format() << endl;
+
+	if (droppedUris.empty() == false)
+	{
+		locationEntry->set_text(droppedUris.front());
+
+		goodDrop = true;
+	}
+
+	context->drag_finish(goodDrop, false, time);
+}
+
 void importDialog::populate_comboboxes(void)
 {
 	labelNameCombobox->append_text(_("None"));
@@ -156,9 +184,9 @@ void importDialog::on_locationEntry_changed()
 	m_locationLength = locationLength;
 
 	// Get 10 URLs like this one
-	QueryHistory history(PinotSettings::getInstance().getHistoryDatabaseName());
+	QueryHistory queryHistory(PinotSettings::getInstance().getHistoryDatabaseName());
 	set<string> suggestedUrls;
-	history.findUrlsLike(fileName, 10, suggestedUrls);
+	queryHistory.findUrlsLike(fileName, 10, suggestedUrls);
 	// Populate the list
 	for (set<string>::iterator urlIter = suggestedUrls.begin();
 		urlIter != suggestedUrls.end(); ++urlIter)
