@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,15 +30,6 @@
 #include <glibmm/miscutils.h>
 #include <glibmm/convert.h>
 #include "config.h"
-extern "C"
-{
-#if DBUS_NUM_VERSION < 1000000
-#define DBUS_API_SUBJECT_TO_CHANGE
-#endif
-#include <dbus/dbus.h>
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-lowlevel.h>
-}
 #include <gtkmm/main.h>
 
 #include "NLS.h"
@@ -50,8 +41,12 @@ extern "C"
 #include "QueryHistory.h"
 #include "ViewHistory.h"
 #include "DownloaderInterface.h"
+#ifdef HAVE_DBUS
+#include "DBusIndex.h"
+#endif
 #include "PinotSettings.h"
 #include "mainWindow.hh"
+#include "prefsWindow.hh"
 
 using namespace std;
 
@@ -60,6 +55,7 @@ static streambuf *g_coutBuf = NULL;
 static streambuf *g_cerrBuf = NULL;
 static struct option g_longOptions[] = {
 	{"help", 0, 0, 'h'},
+	{"preferences", 0, 0, 'p'},
 	{"version", 0, 0, 'v'},
 	{0, 0, 0, 0}
 };
@@ -68,17 +64,8 @@ static void closeAll(void)
 {
 	cout << "Exiting..." << endl;
 
-	// Save the settings
-	PinotSettings &settings = PinotSettings::getInstance();
-	if (settings.save() == false)
-	{
-		cerr << "Couldn't save configuration file" << endl;
-	}
-
 	// Close everything
 	ModuleFactory::unloadModules();
-	Dijon::FilterFactory::unloadFilters();
-	Dijon::HtmlFilter::shutdown();
 
 	// Restore the stream buffers
 	if (g_coutBuf != NULL)
@@ -102,16 +89,68 @@ static void quitAll(int sigNum)
 	Gtk::Main::quit();
 }
 
+static DBusHandlerResult filterHandler(DBusConnection *pConnection, DBusMessage *pMessage, void *pData)
+{
+	// Are we about to be disconnected ?
+	if (dbus_message_is_signal(pMessage, DBUS_INTERFACE_LOCAL, "Disconnected") == TRUE)
+	{
+#ifdef DEBUG
+		cout << "filterHandler: received Disconnected" << endl;
+#endif
+	}
+	else if (dbus_message_is_signal(pMessage, DBUS_INTERFACE_DBUS, "NameOwnerChanged") == TRUE)
+	{
+#ifdef DEBUG
+		cout << "filterHandler: received NameOwnerChanged" << endl;
+#endif
+	}
+	else if (dbus_message_is_signal(pMessage, PINOT_DBUS_SERVICE_NAME, "IndexFlushed") == TRUE)
+	{
+		DBusError error;
+		gboolean onBattery = FALSE;
+		unsigned int docsCount = 0;
+
+#ifdef DEBUG
+		cout << "filterHandler: received IndexFlushed" << endl;
+#endif
+		dbus_error_init(&error);
+		if ((dbus_message_get_args(pMessage, &error,
+			DBUS_TYPE_UINT32, &docsCount,
+			DBUS_TYPE_INVALID) == TRUE))
+		{
+			PinotSettings &settings = PinotSettings::getInstance();
+
+#ifdef DEBUG
+			cout << "filterHandler: reopening index, now with " << docsCount << " documents" << endl;
+#endif
+			IndexInterface *pIndex = settings.getIndex(settings.m_daemonIndexLocation);
+			if (pIndex != NULL)
+			{
+				pIndex->reopen();
+
+				delete pIndex;
+			}
+#ifdef DEBUG
+			cout << "filterHandler: reopened index" << endl;
+#endif
+		}
+		dbus_error_free(&error);
+
+		return DBUS_HANDLER_RESULT_HANDLED;
+	}
+
+	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+
 int main(int argc, char **argv)
 {
 	string prefixDir(PREFIX);
 	Glib::ustring errorMsg;
-	struct sigaction newAction;
 	int longOptionIndex = 0;
-	bool warnAboutVersion = false;
+	bool warnAboutVersion = false, prefsMode = false;
 
 	// Look at the options
-	int optionChar = getopt_long(argc, argv, "hv", g_longOptions, &longOptionIndex);
+	int optionChar = getopt_long(argc, argv, "hpv", g_longOptions, &longOptionIndex);
 	while (optionChar != -1)
 	{
 		switch (optionChar)
@@ -122,9 +161,13 @@ int main(int argc, char **argv)
 					<< "Usage: pinot [OPTIONS]\n\n"
 					<< "Options:\n"
 					<< "  -h, --help		display this help and exit\n"
+					<< "  -p, --preferences		show preferences and exit\n"
 					<< "  -v, --version		output version information and exit\n"
 					<< "\nReport bugs to " << PACKAGE_BUGREPORT << endl;
 				return EXIT_SUCCESS;
+			case 'p':
+				prefsMode = true;
+				break;
 			case 'v':
 				cout << "pinot - " << PACKAGE_STRING << "\n\n" 
 					<< "This is free software.  You may redistribute copies of it under the terms of\n"
@@ -136,7 +179,14 @@ int main(int argc, char **argv)
 		}
 
 		// Next option
-		optionChar = getopt_long(argc, argv, "hv", g_longOptions, &longOptionIndex);
+		optionChar = getopt_long(argc, argv, "hpv", g_longOptions, &longOptionIndex);
+	}
+
+	string programName(argv[0]);
+	if ((programName.length() >= 11) &&
+		(programName.substr(programName.length() - 11) == "pinot-prefs"))
+	{
+		prefsMode = true;
 	}
 
 #if defined(ENABLE_NLS)
@@ -146,19 +196,25 @@ int main(int argc, char **argv)
 #endif //ENABLE_NLS
 
 	// Initialize threads support before doing anything else
-	if (Glib::thread_supported() == false)
-	{
-		Glib::thread_init();
-	}
+	Glib::thread_init();
 	// Initialize the GType and the D-Bus thread system
 	g_type_init();
+#ifdef HAVE_DBUS
 #if DBUS_NUM_VERSION > 1000000
 	dbus_threads_init_default();
 #endif
 	dbus_g_thread_init();
+#endif
 
 	Gtk::Main m(&argc, &argv);
-	Glib::set_application_name("Pinot GTK2 UI");
+	if (prefsMode == false)
+	{
+		Glib::set_application_name("Pinot GTK2 UI");
+	}
+	else
+	{
+		Glib::set_application_name("Pinot Preferences");
+	}
 
 	// This should make Xapian use Flint rather than Quartz
 	Glib::setenv("XAPIAN_PREFER_FLINT", "1");
@@ -201,15 +257,24 @@ int main(int argc, char **argv)
 
 	// This will create the necessary directories on the first run
 	PinotSettings &settings = PinotSettings::getInstance();
+#ifdef HAVE_DBUS
 	// Talk to the daemon through DBus
 	settings.enableClientMode(true);
+#endif
 
-	string confDirectory = PinotSettings::getConfigurationDirectory();
+	string confDirectory(PinotSettings::getConfigurationDirectory());
 	if (chdir(confDirectory.c_str()) == 0)
 	{
 		// Redirect cout and cerr to a file
 		string logFileName = confDirectory;
-		logFileName += "/pinot.log";
+		if (prefsMode == false)
+		{
+			logFileName += "/pinot.log";
+		}
+		else
+		{
+			logFileName += "/pinot-prefs.log";
+		}
 		g_outputFile.open(logFileName.c_str());
 		g_coutBuf = cout.rdbuf();
 		g_cerrBuf = cerr.rdbuf();
@@ -224,8 +289,7 @@ int main(int argc, char **argv)
 		cerr << "Couldn't load MIME settings" << endl;
 	}
 	DownloaderInterface::initialize();
-	// Load tokenizer libraries, if any
-	Dijon::HtmlFilter::initialize();
+	// Load filter libraries, if any
 	Dijon::FilterFactory::loadFilters(string(LIBDIR) + "/pinot/filters");
 	Dijon::FilterFactory::loadFilters(confDirectory + "/filters");
 	// Load backends, if any
@@ -250,24 +314,29 @@ int main(int argc, char **argv)
 	Languages::setIntlName(14, _("Swedish"));
 	Languages::setIntlName(15, _("Turkish"));
 
-	// Load search engines
-	settings.loadSearchEngines(prefixDir + "/share/pinot/engines");
-	settings.loadSearchEngines(confDirectory + "/engines");
 	// Load the settings
-	settings.loadGlobal(string(SYSCONFDIR) + "/pinot/globalconfig.xml");
-	settings.load();
+	settings.load(PinotSettings::LOAD_ALL);
 
 	// Catch interrupts
+#ifdef HAVE_SIGACTION
+	struct sigaction newAction;
 	sigemptyset(&newAction.sa_mask);
 	newAction.sa_flags = 0;
 	newAction.sa_handler = quitAll;
 	sigaction(SIGINT, &newAction, NULL);
 	sigaction(SIGQUIT, &newAction, NULL);
 	sigaction(SIGTERM, &newAction, NULL);
+#else
+	signal(SIGINT, quitAll);
+#ifdef SIGQUIT
+	signal(SIGQUIT, quitAll);
+#endif
+	signal(SIGTERM, quitAll);
+#endif
 
-	// Open this index read-write
+	// Open this index read-write, unless we are in preferences mode
 	bool wasObsoleteFormat = false;
-	if (ModuleFactory::openOrCreateIndex(settings.m_defaultBackend, settings.m_docsIndexLocation, wasObsoleteFormat, false) == false)
+	if (ModuleFactory::openOrCreateIndex(settings.m_defaultBackend, settings.m_docsIndexLocation, wasObsoleteFormat, prefsMode) == false)
 	{
 		errorMsg = _("Couldn't open index");
 		errorMsg += " ";
@@ -310,32 +379,77 @@ int main(int argc, char **argv)
 
 	atexit(closeAll);
 
-	IndexInterface *pIndex = settings.getIndex(settings.m_docsIndexLocation);
-	if (pIndex != NULL)
+	if (prefsMode == false)
 	{
-		string indexVersion(pIndex->getMetadata("version"));
+		GError *pError = NULL;
+		DBusGConnection *pBus = dbus_g_bus_get(DBUS_BUS_SESSION, &pError);
+		DBusConnection *pConnection = NULL;
 
-		// What version is the index at ?
-		if (indexVersion.empty() == true)
+		if (pBus == NULL)
 		{
-			indexVersion = "0.0";
+			if (pError != NULL)
+			{
+				cerr << "Couldn't open bus connection: " << pError->message << endl;
+				if (pError->message != NULL)
+				{
+					cerr << "Error is " << pError->message << endl;
+				}
+				g_error_free(pError);
+			}
 		}
-		// Is an upgrade necessary ?
-		if ((indexVersion < PINOT_INDEX_MIN_VERSION) &&
-			(pIndex->getDocumentsCount() > 0))
+		else
 		{
-			warnAboutVersion = true;
+			pConnection = dbus_g_connection_get_connection(pBus);
 		}
+
+		if (pConnection != NULL)
+		{
+			DBusError error;
+
+			dbus_error_init(&error);
+			dbus_connection_set_exit_on_disconnect(pConnection, FALSE);
+			dbus_connection_setup_with_g_main(pConnection, NULL);
+
+			string rule("type='signal',interface='");
+			rule += PINOT_DBUS_SERVICE_NAME;
+			rule += "'";
+
+			// See signals coming from the daemon
+			dbus_bus_add_match(pConnection, rule.c_str(), &error);
+
+			dbus_connection_add_filter(pConnection,
+				(DBusHandleMessageFunction)filterHandler, NULL, NULL);
+
+			dbus_error_free(&error);
+		}
+
+		IndexInterface *pIndex = settings.getIndex(settings.m_docsIndexLocation);
+		if (pIndex != NULL)
+		{
+			string indexVersion(pIndex->getMetadata("version"));
+
+			// What version is the index at ?
+			if (indexVersion.empty() == true)
+			{
+				indexVersion = "0.0";
+			}
+			// Is an upgrade necessary ?
+			if ((indexVersion < PINOT_INDEX_MIN_VERSION) &&
+				(pIndex->getDocumentsCount() > 0))
+			{
+				warnAboutVersion = true;
+			}
 #ifdef DEBUG
-		cout << "My Web Pages was set to version " << indexVersion << endl;
+			cout << "My Web Pages was set to version " << indexVersion << endl;
 #endif
-		pIndex->setMetadata("version", VERSION);
+			pIndex->setMetadata("version", VERSION);
 
-		delete pIndex;
-	}
-	if (warnAboutVersion == true)
-	{
-		settings.m_warnAboutVersion = warnAboutVersion;
+			delete pIndex;
+		}
+		if (warnAboutVersion == true)
+		{
+			settings.m_warnAboutVersion = warnAboutVersion;
+		}
 	}
 
 	try
@@ -343,13 +457,21 @@ int main(int argc, char **argv)
 		// Set an icon for all windows
 		Gtk::Window::set_default_icon_from_file(prefixDir + "/share/icons/hicolor/48x48/apps/pinot.png");
 
-		// Create and open the main dialog box
-		mainWindow mainBox;
-		if (errorMsg.empty() == false)
+		// Create and open the window
+		if (prefsMode == false)
+		{ 
+			mainWindow mainBox;
+			if (errorMsg.empty() == false)
+			{
+				mainBox.set_status(errorMsg);
+			}
+			m.run(mainBox);
+		}
+		else
 		{
-			mainBox.set_status(errorMsg);
+			prefsWindow prefsBox;
+			m.run(prefsBox);
 		}
-		m.run(mainBox);
 	}
 	catch (const Glib::Exception &e)
 	{
