@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -92,7 +92,7 @@ bool queryDialog::is_separator(const RefPtr<TreeModel>& model, const TreeModel::
 
 		// FIXME: this is extremely hacky !
 		if ((sscanf(rowPath.c_str(), "%u", &rowPos) == 1) &&
-			(rowPos == 10))
+			(rowPos == 12))
 		{
 			return true;
 		}
@@ -113,6 +113,8 @@ void queryDialog::populate_comboboxes()
 	filterCombobox->append_text(_("Title"));
 	filterCombobox->append_text(_("URL"));
 	filterCombobox->append_text(_("Directory"));
+	filterCombobox->append_text(_("In URL"));
+	filterCombobox->append_text(_("Path"));
 	filterCombobox->append_text(_("Language code"));
 	filterCombobox->append_text(_("MIME type"));
 	filterCombobox->append_text(_("MIME class"));
@@ -142,9 +144,9 @@ void queryDialog::populate_comboboxes()
 	string language(m_properties.getStemmingLanguage());
 	for (unsigned int languageNum = 1; languageNum < Languages::m_count; ++languageNum)
 	{
-		string languageName(Languages::getIntlName(languageNum));
+		ustring languageName(Languages::getIntlName(languageNum));
 
-		stemmingCombobox->append_text(to_utf8(languageName));
+		stemmingCombobox->append_text(languageName);
 		// Is this the language we are looking for ?
 		if (language == languageName)
 		{
@@ -295,29 +297,35 @@ void queryDialog::on_addFilterButton_clicked()
 			filter = "dir:/home/xxx";
 			break;
 		case 6:
-			filter = "lang:en";
+			filter = "inurl:file:///home/xxx/yyy.tar.gz";
 			break;
 		case 7:
-			filter = "type:text/plain";
+			filter = "path:Documents";
 			break;
 		case 8:
-			filter = "class:text";
+			filter = "lang:en";
 			break;
 		case 9:
-			filter = "label:New";
+			filter = "type:text/plain";
 			break;
 		case 10:
-			// Separator
+			filter = "class:text";
 			break;
 		case 11:
+			filter = "label:New";
+			break;
+		case 12:
+			// Separator
+			break;
+		case 13:
 			filter = TimeConverter::toYYYYMMDDString(tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday);
 			filter += "..20991231";
 			break;
-		case 12:
+		case 14:
 			filter = TimeConverter::toHHMMSSString(tm->tm_hour, tm->tm_min, tm->tm_sec);
 			filter += "..235959";
 			break;
-		case 13:
+		case 15:
 			filter += "0..10240b";
 			break;
 		default:
