@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005-2008 Fabrice Colin
+ *  Copyright 2005-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,24 +29,90 @@
 #include "Url.h"
 #include "CrawlHistory.h"
 #include "ViewHistory.h"
+#ifdef HAVE_DBUS
 #include "DBusIndex.h"
+#endif
 #include "ModuleFactory.h"
 #include "PinotSettings.h"
 #include "PinotUtils.h"
-#include "WorkerThreads.h"
 #include "statisticsDialog.hh"
 
 using namespace std;
 using namespace Glib;
 using namespace Gtk;
 
+#ifdef HAVE_DBUS
+class DaemonStatusThread : public WorkerThread
+{
+        public:
+                DaemonStatusThread() :
+			WorkerThread(),
+			m_gotStats(false),
+			m_lowDiskSpace(false),
+			m_onBattery(false),
+			m_crawling(false),
+			m_crawledCount(0),
+			m_docsCount(0)
+		{
+		}
+                virtual ~DaemonStatusThread()
+		{
+		}
+
+                virtual std::string getType(void) const
+		{
+			return "DaemonStatusThread";
+		}
+
+		bool m_gotStats;
+		bool m_lowDiskSpace;
+		bool m_onBattery;
+		bool m_crawling;
+		unsigned int m_crawledCount;
+		unsigned int m_docsCount;
+
+        protected:
+                virtual void doWork(void)
+		{
+			if (DBusIndex::getStatistics(m_crawledCount, m_docsCount,
+				m_lowDiskSpace, m_onBattery, m_crawling) == true)
+			{
+				m_gotStats = true;
+			}
+#ifdef DEBUG
+			else cout << "DaemonStatusThread::doWork: failed to get statistics" << endl;
+#endif
+		}
+
+        private:
+                DaemonStatusThread(const DaemonStatusThread &other);
+                DaemonStatusThread &operator=(const DaemonStatusThread &other);
+
+};
+#endif
+
+statisticsDialog::InternalState::InternalState(unsigned int maxIndexThreads, statisticsDialog *pWindow) :
+        ThreadsManager(PinotSettings::getInstance().m_docsIndexLocation, maxIndexThreads),
+	m_getStats(true),
+	m_gettingStats(false),
+	m_lowDiskSpace(false),
+	m_onBattery(false),
+	m_crawling(false)
+{
+        m_onThreadEndSignal.connect(sigc::mem_fun(*pWindow, &statisticsDialog::on_thread_end));
+}
+
+statisticsDialog::InternalState::~InternalState()
+{
+}
+
 statisticsDialog::statisticsDialog() :
 	statisticsDialog_glade(),
 	m_hasErrors(false),
 	m_hasDiskSpace(false),
 	m_hasBattery(false),
 	m_hasCrawl(false),
-	m_getStats(true)
+	m_state(10, this)
 {
 	// Associate the columns model to the engines tree
 	m_refStore = TreeStore::create(m_statsColumns);
@@ -66,11 +132,15 @@ statisticsDialog::statisticsDialog() :
 	// ...and update regularly
 	m_idleConnection = Glib::signal_timeout().connect(sigc::mem_fun(*this,
 		&statisticsDialog::on_activity_timeout), 10000);
+
+	// Connect to threads' finished signal
+	m_state.connect();
 }
 
 statisticsDialog::~statisticsDialog()
 {
 	m_idleConnection.disconnect();
+	m_state.disconnect();
 }
 
 void statisticsDialog::populate(void)
@@ -86,10 +156,14 @@ void statisticsDialog::populate(void)
 	row = *statIter;
 	row[m_statsColumns.m_name] = _("My Web Pages");
 	m_myWebPagesIter = m_refStore->append(statIter->children());
+	row = *m_myWebPagesIter;
+	row[m_statsColumns.m_name] = ustring(_("Checking"));
 	statIter = m_refStore->append(folderIter->children());
 	row = *statIter;
 	row[m_statsColumns.m_name] = _("My Documents");
 	m_myDocumentsIter = m_refStore->append(statIter->children());
+	row = *m_myDocumentsIter;
+	row[m_statsColumns.m_name] = ustring(_("Checking"));
 
 	// Search engines
 	TreeModel::iterator enginesIter = m_refStore->append();
@@ -109,31 +183,51 @@ void statisticsDialog::populate(void)
 	row = *folderIter;
 	row[m_statsColumns.m_name] = _("History");
 	m_viewStatIter = m_refStore->append(folderIter->children());
+	row = *m_viewStatIter;
 	m_crawledStatIter = m_refStore->append(folderIter->children());
+	row = *m_crawledStatIter;
+	populate_history();
 
 	// Daemon
 	m_daemonIter = m_refStore->append();
 	row = *m_daemonIter;
 	row[m_statsColumns.m_name] = _("Daemon");
 	m_daemonProcIter = m_refStore->append(m_daemonIter->children());
+	row = *m_daemonProcIter;
+	row[m_statsColumns.m_name] = ustring(_("Checking"));
 
 	// Expand everything
 	statisticsTreeview->expand_all();
 	TreeModel::Path enginesPath = m_refStore->get_path(enginesIter);
 	statisticsTreeview->collapse_row(enginesPath);
-
-	on_activity_timeout();
 }
 
-bool statisticsDialog::on_activity_timeout(void)
+void statisticsDialog::populate_history(void)
 {
 	CrawlHistory crawlHistory(PinotSettings::getInstance().getHistoryDatabaseName(true));
 	ViewHistory viewHistory(PinotSettings::getInstance().getHistoryDatabaseName());
+	TreeModel::Row row;
+	char countStr[64];
+
+	// Show view statistics
+	unsigned int viewCount = viewHistory.getItemsCount();
+	snprintf(countStr, 64, "%u", viewCount);
+	row = *m_viewStatIter;
+	row[m_statsColumns.m_name] = ustring(_("Viewed")) + " " + countStr + " " + _("results");
+
+	// Show crawler statistics
+	unsigned int crawledFilesCount = crawlHistory.getItemsCount(CrawlHistory::CRAWLED);
+	snprintf(countStr, 64, "%u", crawledFilesCount);
+	row = *m_crawledStatIter;
+	row[m_statsColumns.m_name] = ustring(_("Crawled")) + " " + countStr + " " + _("files");
+}
+
+bool statisticsDialog::on_activity_timeout(void)
+{
 	TreeModel::Row row;
 	std::map<unsigned int, string> sources;
 	string daemonDBusStatus;
 	char countStr[64];
-	bool lowDiskSpace = false, onBattery = false, crawling = false;
 
 	row = *m_myWebPagesIter;
 	IndexInterface *pIndex = PinotSettings::getInstance().getIndex(PinotSettings::getInstance().m_docsIndexLocation);
@@ -169,17 +263,7 @@ bool statisticsDialog::on_activity_timeout(void)
 		row[m_statsColumns.m_name] = _("Unknown error");
 	}
 
-	// Show view statistics
-	unsigned int viewCount = viewHistory.getItemsCount();
-	snprintf(countStr, 64, "%u", viewCount);
-	row = *m_viewStatIter;
-	row[m_statsColumns.m_name] = ustring(_("Viewed")) + " " + countStr + " " + _("results");
-
-	// Show crawler statistics
-	unsigned int crawledFilesCount = crawlHistory.getItemsCount(CrawlHistory::CRAWLED);
-	snprintf(countStr, 64, "%u", crawledFilesCount);
-	row = *m_crawledStatIter;
-	row[m_statsColumns.m_name] = ustring(_("Crawled")) + " " + countStr + " " + _("files");
+	populate_history();
 
 	// Is the daemon still running ?
 	string pidFileName(PinotSettings::getInstance().getConfigurationDirectory() + "/pinot-dbus-daemon.pid");
@@ -199,20 +283,21 @@ bool statisticsDialog::on_activity_timeout(void)
 		// FIXME: check whether it's actually running !
 		row[m_statsColumns.m_name] = ustring(_("Running under PID")) + " " + countStr;
 
-		if (m_getStats == true)
+#ifdef HAVE_DBUS
+		if ((m_state.m_getStats == true) &&
+			(m_state.m_gettingStats == false))
 		{
-			unsigned int crawledCount = 0, docsCount = 0;
+			m_state.m_gettingStats = true;
 
-			if (DBusIndex::getStatistics(crawledCount, docsCount, lowDiskSpace, onBattery, crawling) == false)
+			DaemonStatusThread *pThread = new DaemonStatusThread();
+
+			if (m_state.start_thread(pThread, false) == false)
 			{
-#ifdef DEBUG
-				cout << "statisticsDialog::on_activity_timeout: failed to get statistics" << endl;
-#endif
-
-				// Don't try again
-				m_getStats = false;
+				delete pThread;
+				m_state.m_getStats = m_state.m_gettingStats = false;
 			}
 		}
+#endif
 	}
 	else
 	{
@@ -231,7 +316,7 @@ bool statisticsDialog::on_activity_timeout(void)
 	}
 
 	// Show status
-	if (lowDiskSpace == true)
+	if (m_state.m_lowDiskSpace == true)
 	{
 		if (m_hasDiskSpace == false)
 		{
@@ -248,7 +333,7 @@ bool statisticsDialog::on_activity_timeout(void)
 
 		m_hasDiskSpace = false;
 	}
-	if (onBattery == true)
+	if (m_state.m_onBattery == true)
 	{
 		if (m_hasBattery == false)
 		{
@@ -265,7 +350,7 @@ bool statisticsDialog::on_activity_timeout(void)
 
 		m_hasBattery = false;
 	}
-	if (crawling == true)
+	if (m_state.m_crawling == true)
 	{
 		if (m_hasCrawl == false)
 		{
@@ -284,6 +369,8 @@ bool statisticsDialog::on_activity_timeout(void)
 		m_hasCrawl = false;
 	}
 
+	CrawlHistory crawlHistory(PinotSettings::getInstance().getHistoryDatabaseName(true));
+
 	// Show errors
 	crawlHistory.getSources(sources);
 	for (std::map<unsigned int, string>::iterator sourceIter = sources.begin();
@@ -292,6 +379,7 @@ bool statisticsDialog::on_activity_timeout(void)
 		unsigned int sourceNum(sourceIter->first);
 		set<string> errors;
 		time_t latestErrorDate = 0;
+		unsigned int currentOffset = 0;
 
 		std::map<unsigned int, time_t>::const_iterator dateIter = m_latestErrorDates.find(sourceNum);
 		if (dateIter != m_latestErrorDates.end())
@@ -301,8 +389,9 @@ bool statisticsDialog::on_activity_timeout(void)
 
 		// Did any error occur on this source ?
 		unsigned int errorCount = crawlHistory.getSourceItems(sourceNum,
-			CrawlHistory::ERROR, errors, latestErrorDate);
-		if ((errorCount > 0) &&
+			CrawlHistory::CRAWL_ERROR, errors, currentOffset, currentOffset + 100,
+			latestErrorDate);
+		while ((errorCount > 0) &&
 			(errors.empty() == false))
 		{
 			// Add an errors row
@@ -355,6 +444,16 @@ bool statisticsDialog::on_activity_timeout(void)
 			// Expand errors
 			TreeModel::Path errPath = m_refStore->get_path(m_errorsTopIter);
 			statisticsTreeview->expand_to_path(errPath);
+
+			// Next
+			if (errors.size() < 100)
+			{
+				break;
+			}
+			currentOffset += 100;
+			errorCount = crawlHistory.getSourceItems(sourceNum,
+				CrawlHistory::CRAWL_ERROR, errors, currentOffset, 100, // currentOffset + 100,
+				latestErrorDate);
 		}
 
 		// The next check will ignore errors older than this
@@ -364,3 +463,56 @@ bool statisticsDialog::on_activity_timeout(void)
 	return true;
 }
 
+void statisticsDialog::on_thread_end(WorkerThread *pThread)
+{
+	ustring status;
+	bool success = true;
+
+	if (pThread == NULL)
+	{
+		return;
+	}
+
+	// Did the thread fail ?
+	status = pThread->getStatus();
+	if (status.empty() == false)
+	{
+#ifdef DEBUG
+		cout << "statisticsDialog::on_thread_end: " << status << endl;
+#endif
+		success = false;
+	}
+
+	// What type of thread was it ?
+	string type = pThread->getType();
+#ifdef HAVE_DBUS
+	if (type == "DaemonStatusThread")
+	{
+		m_state.m_gettingStats = false;
+
+		// Did it succeed ?
+		if (success == true)
+		{
+			DaemonStatusThread *pStatusThread = dynamic_cast<DaemonStatusThread*>(pThread);
+			if (pStatusThread != NULL)
+			{
+				// Yes, it did
+				m_state.m_getStats = pStatusThread->m_gotStats;
+				m_state.m_lowDiskSpace = pStatusThread->m_lowDiskSpace;
+				m_state.m_onBattery = pStatusThread->m_onBattery;
+				m_state.m_crawling = pStatusThread->m_crawling;
+#ifdef DEBUG
+				cout << "statisticsDialog::on_thread_end: refreshed stats" << endl;
+#endif
+			}
+		}
+	}
+#endif
+
+	// Delete the thread
+	delete pThread;
+
+	// We might be able to run a queued action
+	m_state.pop_queue();
+}
+
