@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,24 +16,53 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_MMAP
 #include <sys/mman.h>
+#endif
+#ifdef HAVE_ATTR_XATTR_H
+#include <attr/xattr.h>
+#endif
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <iostream>
+#include <set>
 
 #include "Document.h"
 #include "TimeConverter.h"
+#include "Memory.h"
 
 using std::cout;
 using std::cerr;
 using std::endl;
 using std::string;
+using std::set;
+
+#ifdef HAVE_ATTR_XATTR_H
+static char *getXAttr(int fd, const string &attrName)
+{
+	ssize_t attrSize = fgetxattr(fd, attrName.c_str(), NULL, 0);
+
+	if (attrSize > 0)
+	{
+		char *pAttr = new char[attrSize];
+
+		if ((pAttr != NULL) &&
+			(fgetxattr(fd, attrName.c_str(), pAttr, attrSize) > 0))
+		{
+			return pAttr;
+		}
+	}
+
+	return NULL;
+}
+#endif
 
 Document::Document() :
 	DocumentInfo(),
@@ -115,7 +144,7 @@ bool Document::setData(const char *data, unsigned int length)
 	// Discard existing data
 	resetData();
 
-	m_pData = (char *)malloc(sizeof(char) * (length + 1));
+	m_pData = Memory::allocateBuffer(length + 1);
 	if (m_pData != NULL)
 	{
 		memcpy(m_pData, data, length);
@@ -132,10 +161,9 @@ bool Document::setData(const char *data, unsigned int length)
 bool Document::setDataFromFile(const string &fileName)
 {
 	struct stat fileStat;
-#ifndef O_NOATIME
 	int openFlags = O_RDONLY;
-#else
-	int openFlags = O_RDONLY|O_NOATIME;
+#ifdef O_CLOEXEC
+	openFlags = openFlags|O_CLOEXEC;
 #endif
 
 	if (fileName.empty() == true)
@@ -164,13 +192,17 @@ bool Document::setDataFromFile(const string &fileName)
 	}
 
 	// Open the file in read-only mode
+#ifdef O_NOATIME
+	int fd = open(fileName.c_str(), openFlags|O_NOATIME);
+#else
 	int fd = open(fileName.c_str(), openFlags);
+#endif
 #ifdef O_NOATIME
 	if ((fd < 0) &&
 		(errno == EPERM))
 	{
 		// Try again
-		fd = open(fileName.c_str(), O_RDONLY);
+		fd = open(fileName.c_str(), openFlags);
 	}
 #endif
 	if (fd < 0)
@@ -178,24 +210,112 @@ bool Document::setDataFromFile(const string &fileName)
 		cerr << "Document::setDataFromFile: " << fileName << " couldn't be opened" << endl;
 		return false;
 	}
+#ifndef O_CLOEXEC
+	int fdFlags = fcntl(fd, F_GETFD);
+	fcntl(fd, F_SETFD, fdFlags|FD_CLOEXEC);
+#endif
 
 	// Discard existing data
 	resetData();
 
+#ifdef HAVE_MMAP
 	// Request a private mapping of the whole file
-	void *mapSpace = mmap(NULL, fileStat.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
+	void *mapSpace = mmap(NULL, (size_t)fileStat.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
 	if (mapSpace != MAP_FAILED)
 	{
 		m_pData = (char*)mapSpace;
 		m_dataLength = fileStat.st_size;
-		setTimestamp(TimeConverter::toTimestamp(fileStat.st_mtime));
-		setSize(fileStat.st_size);
 		m_isMapped = true;
+#ifdef HAVE_MADVISE
+		if (madvise(mapSpace, (size_t)fileStat.st_size, MADV_SEQUENTIAL) != 0)
+		{
+#ifdef DEBUG
+			cout << "Document::setDataFromFile: ignored memory advice" << endl;
+#endif
+		}
+#endif
 	}
-	else
+	else cerr << "Document::setDataFromFile: mapping failed" << endl;
+#else
+	m_pData = Memory::allocateBuffer(fileStat.st_size + 1);
+	if (m_pData != NULL)
 	{
-		cerr << "Document::setDataFromFile: mapping failed" << endl;
+		if (read(fd, (void*)m_pData, fileStat.st_size) == fileStat.st_size)
+		{
+			m_pData[fileStat.st_size] = '\0';
+			m_dataLength = (unsigned int)fileStat.st_size;
+		}
+		else
+		{
+			Memory::freeBuffer(m_pData, fileStat.st_size + 1);
+			m_pData = NULL;
+		}
 	}
+	else cerr << "Document::setDataFromFile: reading failed" << endl;
+#endif
+
+	setTimestamp(TimeConverter::toTimestamp(fileStat.st_mtime));
+	setSize(fileStat.st_size);
+
+#ifdef HAVE_ATTR_XATTR_H
+	// Any extended attributes ?
+	ssize_t listSize = flistxattr(fd, NULL, 0);
+	if (listSize > 0)
+	{
+		char *pList = new char[listSize];
+
+		if (flistxattr(fd, pList, listSize) > 0)
+		{
+			set<string> labels;
+			string attrList(pList, listSize);
+			string::size_type startPos = 0, endPos = attrList.find('\0');
+
+			while (endPos != string::npos)
+			{
+				string attrName(attrList.substr(startPos, endPos - startPos));
+				char *pAttr = NULL;
+
+				// FIXME: support common attributes defined at
+				// http://www.freedesktop.org/wiki/CommonExtendedAttributes
+				if (attrName == "user.mime_type")
+				{
+					pAttr = getXAttr(fd, attrName);
+					if (pAttr != NULL)
+					{
+						// Set the MIME type
+						setType(pAttr);
+					}
+				}
+
+				if (pAttr != NULL)
+				{
+#ifdef DEBUG
+					cout << "Document::setDataFromFile: xattr " << attrName << "=" << pAttr << endl;
+#endif
+					delete[] pAttr;
+				}
+
+				// Next
+				startPos = endPos + 1;
+				if (startPos < listSize)
+				{
+					endPos = attrList.find('\0', startPos);
+				}
+				else
+				{
+					endPos = string::npos;
+				}
+			}
+
+			if (labels.empty() == false)
+			{
+				setLabels(labels);
+			}
+		}
+
+		delete[] pList;
+	}
+#endif
 
 	// Close the file
 	if (close(fd) == -1)
@@ -223,13 +343,23 @@ void Document::resetData(void)
 		if (m_isMapped == false)
 		{
 			// Free
-			free(m_pData);
+			Memory::freeBuffer(m_pData, m_dataLength + 1);
 		}
+#ifdef HAVE_MMAP
 		else
 		{
+#ifdef HAVE_MADVISE
+			if (madvise((void*)m_pData, (size_t)m_dataLength, MADV_DONTNEED) != 0)
+			{
+#ifdef DEBUG
+				cout << "Document::resetData: ignored memory advice" << endl;
+#endif
+			}
+#endif
 			// Unmap
-			munmap((void*)m_pData, m_dataLength);
+			munmap((void*)m_pData, (size_t)m_dataLength);
 		}
+#endif
 	}
 
 	m_pData = NULL;
