@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -25,19 +25,27 @@
 #include <map>
 #include <set>
 #include <vector>
+#ifdef USE_GIO
+#include <gio/gio.h>
+#endif
 
 #include "Url.h"
+#include "Visibility.h"
 
 /** MIMEAction stores useful information extracted from desktop files.
   * The desktop files format is defined by
   * http://standards.freedesktop.org/desktop-entry-spec/latest/
   */
-class MIMEAction
+class PINOT_EXPORT MIMEAction
 {
 	public:
 		MIMEAction();
 		MIMEAction(const std::string &name, const std::string &cmdLine);
+#ifdef USE_GIO
+		MIMEAction(GAppInfo *pAppInfo);
+#else
 		MIMEAction(const std::string &desktopFile);
+#endif
 		MIMEAction(const MIMEAction &other);
 		virtual ~MIMEAction();
 
@@ -45,19 +53,26 @@ class MIMEAction
 
 		MIMEAction &operator=(const MIMEAction &other);
 
+#ifndef USE_GIO
 		void load(void);
 		void parseExec(void);
+#endif
 
 		bool m_multipleArgs;
 		bool m_localOnly;
 		std::string m_name;
 		std::string m_location;
 		std::string m_exec;
+#ifdef USE_GIO
+		GAppInfo *m_pAppInfo;
+#else
 		std::string m_icon;
 		std::string m_device;
+#endif
 
 };
 
+#ifndef USE_GIO 
 /** Caching of MIME type information.
   * A cache can be reloaded as required.
   */
@@ -85,11 +100,12 @@ class MIMECache
 			std::map<std::string, MIMEAction> &loadedActions);
 
 };
+#endif
 
 /**
   * Utility class to get a file's MIME type and the default application associated with it.
   */
-class MIMEScanner
+class PINOT_EXPORT MIMEScanner
 {
 	public:
 		~MIMEScanner();
@@ -109,8 +125,13 @@ class MIMEScanner
 		/// Finds out the given URL's MIME type.
 		static std::string scanUrl(const Url &urlObj);
 
+		/// Finds out the given data buffer's MIME type.
+		static std::string scanData(const char *pData, unsigned int length);
+
 		/// Gets parent MIME types.
-		static bool getParentTypes(const std::string &mimeType, std::set<std::string> &parentMimeTypes);
+		static bool getParentTypes(const std::string &mimeType,
+			const std::set<std::string> &allTypes,
+			std::set<std::string> &parentMimeTypes);
 
 		/// Adds a user-defined action for the given type.
 		static void addDefaultAction(const std::string &mimeType, const MIMEAction &typeAction);
@@ -119,12 +140,14 @@ class MIMEScanner
 		static bool getDefaultActions(const std::string &mimeType, std::vector<MIMEAction> &typeActions);
 
 	protected:
+#ifndef USE_GIO
 		/// Mutex to protect access to xdgmime.
 		static pthread_mutex_t m_xdgMutex;
 		/// Lock to protect access to caches.
 		static pthread_rwlock_t m_cachesLock;
 		/// MIME type caches, ordered by decreasing priority.
 		static std::list<MIMECache> m_caches;
+#endif
 
 		MIMEScanner();
 
