@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,10 +16,17 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <stdio.h>
+#include <unistd.h>
+#include <iostream>
+#include <glibmm/miscutils.h>
+
 #include "StringManip.h"
 #include "Url.h"
 
 using std::string;
+using std::cout;
+using std::endl;
 
 static const int g_rfc2396Encoded[] = {
 	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  /* 0x00 - 0x0f */
@@ -45,8 +52,42 @@ Url::Url(const string &url)
 	parse(url);
 }
 
-Url::~Url()
+Url::Url(const string &path, const string &parentPath)
 {
+	string absoluteUrl;
+
+	// Is this a relative path ?
+	string::size_type pos = path.find("://");
+	if ((pos == string::npos) &&
+		(Glib::path_is_absolute(path) == false))
+	{
+		if (parentPath.empty() == true)
+		{
+			char *pCurrentDir = (char *)malloc(sizeof(char) * PATH_MAX);
+
+			if (pCurrentDir != NULL)
+			{
+				if (getcwd(pCurrentDir, PATH_MAX) != NULL)
+				{
+					absoluteUrl = Url::resolvePath(pCurrentDir, path);
+				}
+				free(pCurrentDir);
+			}
+		}
+		else
+		{
+			absoluteUrl = Url::resolvePath(parentPath, path);
+		}
+	}
+
+	if (absoluteUrl.empty() == false)
+	{
+		parse(absoluteUrl);
+	}
+	else
+	{
+		parse(path);
+	}
 }
 
 Url::Url(const Url &other) :
@@ -60,6 +101,10 @@ Url::Url(const Url &other) :
 {
 }
 
+Url::~Url()
+{
+}
+
 Url& Url::operator=(const Url& other)
 {
 	if (this != &other)
@@ -225,9 +270,7 @@ void Url::parse(const string &url)
 
 bool Url::isLocal(const string &protocol) const
 {
-	if ((protocol == "file") ||
-		(protocol == "mailbox") ||
-		(protocol == "xapian"))
+	if (protocol == "file")
 	{
 		return true;
 	}
@@ -444,6 +487,70 @@ string Url::unescapeUrl(const string &escapedUrl)
 	return unescapedUrl;
 }
 
+/// Resolves a path.
+string Url::resolvePath(const string &currentDir, const string &location)
+{
+	string currentLocation(currentDir);
+	string::size_type prevSlashPos = 0, slashPos = location.find('/');
+
+	if (currentDir.empty() == true)
+	{
+		return "";
+	}
+
+	while (slashPos != string::npos)
+	{
+		string path(location.substr(prevSlashPos, slashPos - prevSlashPos));
+
+		if (path == "..")
+		{
+			string upDir(Glib::path_get_dirname(currentLocation));
+			currentLocation = upDir;
+		}
+		else if (path != ".")
+		{
+			currentLocation += "/";
+			currentLocation += path;
+		}
+#ifdef DEBUG
+		cout << "Url::resolvePath: partially resolved to " << currentLocation << endl;
+#endif
+
+		if (slashPos + 1 >= location.length())
+		{
+			// Nothing behind
+			prevSlashPos = string::npos;
+			break;
+		}
+
+		// Next
+		prevSlashPos = slashPos + 1;
+		slashPos = location.find('/', prevSlashPos);
+	}
+
+	// Remainder
+	if (prevSlashPos != string::npos)
+	{
+		string path(location.substr(prevSlashPos));
+
+		if (path == "..")
+		{
+			string upDir(Glib::path_get_dirname(currentLocation));
+			currentLocation = upDir;
+		}
+		else if (path != ".")
+		{
+			currentLocation += "/";
+			currentLocation += path;
+		}
+	}
+#ifdef DEBUG
+	cout << "Url::resolvePath: resolved to " << currentLocation << endl;
+#endif
+
+	return currentLocation;
+}
+
 string Url::getProtocol(void) const
 {
 	return m_protocol;
